@@ -21,6 +21,8 @@ import (
 	"math"
 	"path/filepath"
 
+	"github.com/dolthub/dolt/go/store/types"
+
 	"github.com/dolthub/dolt/go/cmd/dolt/cli"
 	"github.com/dolthub/dolt/go/cmd/dolt/errhand"
 	"github.com/dolthub/dolt/go/libraries/doltcore/dbfactory"
@@ -56,6 +58,10 @@ func (cmd InspectCmd) Description() string {
 	return "Inspects a Dolt Database and collects stats."
 }
 
+func (cmd InspectCmd) GatedForNBF(nbf *types.NomsBinFormat) bool {
+	return types.IsFormat_DOLT_1(nbf)
+}
+
 // CreateMarkdown creates a markdown file containing the helptext for the command at the given path
 func (cmd InspectCmd) CreateMarkdown(wr io.Writer, commandStr string) error {
 	return nil
