@@ -32,6 +32,9 @@ type RowDiffer interface {
 	// Start starts the RowDiffer.
 	Start(ctx context.Context, from, to types.Map)
 
+	// StartWithRange starts the RowDiffer with the specified range
+	StartWithRange(ctx context.Context, from, to types.Map, start types.Value, inRange types.ValueInRange)
+
 	// GetDiffs returns the requested number of diff.Differences, or times out.
 	GetDiffs(numDiffs int, timeout time.Duration) ([]*diff.Difference, bool, error)
 
@@ -298,6 +301,10 @@ var _ RowDiffer = &EmptyRowDiffer{}
 func (e EmptyRowDiffer) Start(ctx context.Context, from, to types.Map) {
 }
 
+func (e EmptyRowDiffer) StartWithRange(ctx context.Context, from, to types.Map, start types.Value, inRange types.ValueInRange) {
+
+}
+
 func (e EmptyRowDiffer) GetDiffs(numDiffs int, timeout time.Duration) ([]*diff.Difference, bool, error) {
 	return nil, false, nil
 }
