@@ -17,7 +17,6 @@ package doltdb
 import (
 	"context"
 	"errors"
-	"fmt"
 	"regexp"
 	"strings"
 	"unicode"
@@ -117,16 +116,50 @@ func (t *Table) SetConflicts(ctx context.Context, schemas conflict.ConflictSchem
 
 // GetConflicts returns a map built from ValueReadWriter when there are no conflicts in table.
 func (t *Table) GetConflicts(ctx context.Context) (conflict.ConflictSchema, durable.ConflictIndex, error) {
+	if t.Format() == types.Format_DOLT_1 {
+		panic("should use artifacts")
+	}
+
 	return t.table.GetConflicts(ctx)
 }
 
 // HasConflicts returns true if this table contains merge conflicts.
 func (t *Table) HasConflicts(ctx context.Context) (bool, error) {
+	if t.Format() == types.Format_DOLT_1 {
+		art, err := t.GetArtifacts(ctx)
+		if err != nil {
+			return false, err
+		}
+
+		return art.HasConflicts(ctx)
+	}
 	return t.table.HasConflicts(ctx)
 }
 
+// GetArtifacts returns the merge artifacts for this table.
+func (t *Table) GetArtifacts(ctx context.Context) (durable.ArtifactIndex, error) {
+	return t.table.GetArtifacts(ctx)
+}
+
+// SetArtifacts sets the merge artifacts for this table.
+func (t *Table) SetArtifacts(ctx context.Context, artifacts durable.ArtifactIndex) (*Table, error) {
+	table, err := t.table.SetArtifacts(ctx, artifacts)
+	if err != nil {
+		return nil, err
+	}
+	return &Table{table: table}, nil
+}
+
 // NumRowsInConflict returns the number of rows with merge conflicts for this table.
 func (t *Table) NumRowsInConflict(ctx context.Context) (uint64, error) {
+	if t.Format() == types.Format_DOLT_1 {
+		artIdx, err := t.table.GetArtifacts(ctx)
+		if err != nil {
+			return 0, err
+		}
+		return artIdx.ConflictCount(ctx)
+	}
+
 	ok, err := t.table.HasConflicts(ctx)
 	if err != nil {
 		return 0, err
@@ -145,6 +178,30 @@ func (t *Table) NumRowsInConflict(ctx context.Context) (uint64, error) {
 
 // ClearConflicts deletes all merge conflicts for this table.
 func (t *Table) ClearConflicts(ctx context.Context) (*Table, error) {
+	if t.Format() == types.Format_DOLT_1 {
+		return t.clearArtifactConflicts(ctx)
+	}
+
+	return t.clearConflicts(ctx)
+}
+
+func (t *Table) clearArtifactConflicts(ctx context.Context) (*Table, error) {
+	artIdx, err := t.table.GetArtifacts(ctx)
+	if err != nil {
+		return nil, err
+	}
+	artIdx, err = artIdx.ClearConflicts(ctx)
+	if err != nil {
+		return nil, err
+	}
+	table, err := t.table.SetArtifacts(ctx, artIdx)
+	if err != nil {
+		return nil, err
+	}
+	return &Table{table: table}, nil
+}
+
+func (t *Table) clearConflicts(ctx context.Context) (*Table, error) {
 	table, err := t.table.ClearConflicts(ctx)
 	if err != nil {
 		return nil, err
@@ -432,37 +489,6 @@ func (t *Table) RenameIndexRowData(ctx context.Context, oldIndexName, newIndexNa
 	return t.SetIndexSet(ctx, indexes)
 }
 
-// VerifyIndexRowData verifies that the index with the given name's data matches what the index expects.
-func (t *Table) VerifyIndexRowData(ctx context.Context, indexName string) error {
-	sch, err := t.GetSchema(ctx)
-	if err != nil {
-		return err
-	}
-
-	index := sch.Indexes().GetByName(indexName)
-	if index == nil {
-		return fmt.Errorf("index `%s` does not exist", indexName)
-	}
-
-	indexes, err := t.GetIndexSet(ctx)
-	if err != nil {
-		return err
-	}
-
-	idx, err := indexes.GetIndex(ctx, sch, indexName)
-	if err != nil {
-		return err
-	}
-
-	im := durable.NomsMapFromIndex(idx)
-	iter, err := im.Iterator(ctx)
-	if err != nil {
-		return err
-	}
-
-	return index.VerifyMap(ctx, iter, im.Format())
-}
-
 // GetAutoIncrementValue returns the current AUTO_INCREMENT value for this table.
 func (t *Table) GetAutoIncrementValue(ctx context.Context) (uint64, error) {
 	return t.table.GetAutoIncrement(ctx)
