@@ -58,17 +58,6 @@ type Remote struct {
 	dialer     dbfactory.GRPCDialProvider
 }
 
-func GetRemote(ctx context.Context, remoteName, remoteUrl string, params map[string]string, dialer dbfactory.GRPCDialProvider) (Remote, *doltdb.DoltDB, error) {
-	r := NewRemote(remoteName, remoteUrl, params, dialer)
-	ddb, err := r.GetRemoteDB(ctx, types.Format_Default)
-
-	if err != nil {
-		return NoRemote, nil, err
-	}
-
-	return r, ddb, nil
-}
-
 func NewRemote(name, url string, params map[string]string, dialer dbfactory.GRPCDialProvider) Remote {
 	return Remote{name, url, []string{"refs/heads/*:refs/remotes/" + name + "/*"}, params, dialer}
 }
