@@ -16,14 +16,17 @@ package merge
 
 import (
 	"context"
+	"encoding/json"
 
 	"golang.org/x/sync/errgroup"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb/durable"
 	"github.com/dolthub/dolt/go/libraries/doltcore/schema"
+	"github.com/dolthub/dolt/go/store/hash"
 	"github.com/dolthub/dolt/go/store/pool"
 	"github.com/dolthub/dolt/go/store/prolly"
+	"github.com/dolthub/dolt/go/store/prolly/shim"
 	"github.com/dolthub/dolt/go/store/prolly/tree"
 	"github.com/dolthub/dolt/go/store/types"
 	"github.com/dolthub/dolt/go/store/val"
@@ -31,7 +34,7 @@ import (
 
 type mergeResult struct {
 	tbl   *doltdb.Table
-	cons  durable.ConflictIndex
+	arts  durable.ArtifactIndex
 	stats *MergeStats
 }
 
@@ -47,9 +50,19 @@ type mergeResult struct {
 // entries are set to values consistent the cell-wise merge result. When the
 // root and merge secondary indexes are merged, they will produce entries
 // consistent with the primary row data.
-func mergeTableData(ctx context.Context, vrw types.ValueReadWriter, postMergeSchema, rootSchema, mergeSchema, ancSchema schema.Schema, tbl, mergeTbl, tableToUpdate *doltdb.Table, ancRows durable.Index, ancIndexSet durable.IndexSet) (mergeResult, error) {
+func mergeTableData(
+	ctx context.Context,
+	vrw types.ValueReadWriter,
+	postMergeSchema, rootSchema, mergeSchema, ancSchema schema.Schema,
+	tbl, mergeTbl, tableToUpdate *doltdb.Table,
+	ancRows durable.Index,
+	ancIndexSet durable.IndexSet,
+	cmHash hash.Hash,
+	mergeTblHash, ancTblHash hash.Hash) (mergeResult, error) {
 	group, gCtx := errgroup.WithContext(ctx)
 
+	stats := &MergeStats{Operation: TableModified}
+
 	indexEdits := make(chan indexEdit, 128)
 	conflicts := make(chan confVals, 128)
 	var updatedTable *doltdb.Table
@@ -57,7 +70,6 @@ func mergeTableData(ctx context.Context, vrw types.ValueReadWriter, postMergeSch
 
 	group.Go(func() error {
 		var err error
-		// TODO (dhruv): update this function definition to return any conflicts
 		updatedTable, mergedData, err = mergeProllyRowData(gCtx, postMergeSchema, rootSchema, mergeSchema, ancSchema, tbl, mergeTbl, tableToUpdate, ancRows, indexEdits, conflicts)
 		if err != nil {
 			return err
@@ -84,13 +96,19 @@ func mergeTableData(ctx context.Context, vrw types.ValueReadWriter, postMergeSch
 		return err
 	})
 
-	confIdx, err := durable.NewEmptyConflictIndex(ctx, vrw, rootSchema, mergeSchema, ancSchema)
+	artIdx, err := tbl.GetArtifacts(ctx)
+	if err != nil {
+		return mergeResult{}, err
+	}
+	artM := durable.ProllyMapFromArtifactIndex(artIdx)
+	artifactEditor := artM.Editor()
+	kd, vd := artM.Descriptors()
+	p, err := newConflictProcessor(artM.Pool(), kd, vd, cmHash, ancTblHash, mergeTblHash)
 	if err != nil {
 		return mergeResult{}, err
 	}
-	confEditor := durable.ProllyMapFromConflictIndex(confIdx).Editor()
 	group.Go(func() error {
-		return processConflicts(ctx, conflicts, confEditor)
+		return p.process(gCtx, stats, conflicts, artifactEditor)
 	})
 
 	err = group.Wait()
@@ -107,11 +125,11 @@ func mergeTableData(ctx context.Context, vrw types.ValueReadWriter, postMergeSch
 		return mergeResult{}, err
 	}
 
-	confMap, err := confEditor.Flush(ctx)
+	artifactMap, err := artifactEditor.Flush(ctx)
 	if err != nil {
 		return mergeResult{}, err
 	}
-	confIdx = durable.ConflictIndexFromProllyMap(confMap)
+	artIdx = durable.ArtifactIndexFromProllyMap(artifactMap)
 
 	updatedTable, err = mergeProllySecondaryIndexes(ctx, vrw, postMergeSchema, rootSchema, mergeSchema, ancSchema, mergedData, tbl, mergeTbl, updatedTable, ancIndexSet)
 	if err != nil {
@@ -121,8 +139,8 @@ func mergeTableData(ctx context.Context, vrw types.ValueReadWriter, postMergeSch
 	// TODO (dhruv): populate merge stats
 	return mergeResult{
 		tbl:   updatedTable,
-		cons:  confIdx,
-		stats: &MergeStats{Operation: TableModified},
+		arts:  artIdx,
+		stats: stats,
 	}, nil
 }
 
@@ -231,7 +249,7 @@ func newValueMerger(merged, leftSch, rightSch, baseSch schema.Schema, syncPool p
 
 	return &valueMerger{
 		numCols:      n,
-		vD:           prolly.ValueDescriptorFromSchema(merged),
+		vD:           shim.ValueDescriptorFromSchema(merged),
 		leftMapping:  leftMapping,
 		rightMapping: rightMapping,
 		baseMapping:  baseMapping,
@@ -308,7 +326,37 @@ func (m *valueMerger) processColumn(i int, left, right, base val.Tuple) ([]byte,
 	}
 }
 
-func processConflicts(ctx context.Context, conflictChan chan confVals, editor prolly.ConflictEditor) error {
+type conflictProcessor struct {
+	cmHash       []byte
+	metadataJson []byte
+	keyBD, valBD *val.TupleBuilder
+	pool         pool.BuffPool
+}
+
+func newConflictProcessor(pool pool.BuffPool, artKD, artVD val.TupleDesc, cmHash, baseTblHash, theirTblHash hash.Hash) (*conflictProcessor, error) {
+	h := make([]byte, 20)
+	copy(h, cmHash[:])
+	m := prolly.ConflictMetadata{
+		BaseTblHash:  make([]byte, 20),
+		TheirTblHash: make([]byte, 20),
+	}
+	copy(m.BaseTblHash, baseTblHash[:])
+	copy(m.TheirTblHash, theirTblHash[:])
+	data, err := json.Marshal(m)
+	if err != nil {
+		return nil, err
+	}
+	p := conflictProcessor{
+		cmHash:       h,
+		metadataJson: data,
+		keyBD:        val.NewTupleBuilder(artKD),
+		valBD:        val.NewTupleBuilder(artVD),
+		pool:         pool,
+	}
+	return &p, nil
+}
+
+func (p *conflictProcessor) process(ctx context.Context, stats *MergeStats, conflictChan chan confVals, artEditor prolly.ArtifactsEditor) error {
 OUTER:
 	for {
 		select {
@@ -316,7 +364,12 @@ OUTER:
 			if !ok {
 				break OUTER
 			}
-			err := editor.Add(ctx, conflict.key, conflict.ourVal, conflict.theirVal, conflict.baseVal)
+			stats.Conflicts++
+			k, v, err := p.buildConflictArtifact(conflict.key)
+			if err != nil {
+				return err
+			}
+			err = artEditor.Add(ctx, k, v)
 			if err != nil {
 				return err
 			}
@@ -327,3 +380,17 @@ OUTER:
 
 	return nil
 }
+
+func (p *conflictProcessor) buildConflictArtifact(key val.Tuple) (k, v val.Tuple, err error) {
+	for i := 0; i < key.Count(); i++ {
+		p.keyBD.PutRaw(i, key.GetField(i))
+	}
+	p.keyBD.PutAddress(key.Count(), p.cmHash)
+	p.keyBD.PutUint8(key.Count()+1, uint8(prolly.ArtifactTypeConflict))
+	k = p.keyBD.Build(p.pool)
+
+	p.valBD.PutJSON(0, p.metadataJson)
+	v = p.valBD.Build(p.pool)
+
+	return
+}
