@@ -26,8 +26,11 @@ import (
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
 	"github.com/dolthub/dolt/go/libraries/doltcore/env"
 	"github.com/dolthub/dolt/go/libraries/doltcore/env/actions"
+	"github.com/dolthub/dolt/go/libraries/doltcore/ref"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dsess"
+	"github.com/dolthub/dolt/go/libraries/doltcore/sqlserver"
 	"github.com/dolthub/dolt/go/libraries/utils/argparser"
+	"github.com/dolthub/dolt/go/libraries/utils/filesys"
 )
 
 const DoltBranchFuncName = "dolt_branch"
@@ -91,50 +94,130 @@ func DoDoltBranch(ctx *sql.Context, args []string) (int, error) {
 
 	switch {
 	case apr.Contains(cli.CopyFlag):
-		err = makeACopyOfBranch(ctx, dbData, apr)
-		if err != nil {
-			return 1, err
-		}
+		err = copyBranch(ctx, dbData, apr)
 	case apr.Contains(cli.MoveFlag):
-		return 1, errors.New("Renaming a branch is not supported.")
-	case apr.Contains(cli.DeleteFlag):
-		return 1, errors.New("Deleting branches is not supported.")
-	case apr.Contains(cli.DeleteForceFlag):
-		return 1, errors.New("Deleting branches is not supported.")
+		err = renameBranch(ctx, dbData, apr)
+	case apr.Contains(cli.DeleteFlag), apr.Contains(cli.DeleteForceFlag):
+		err = deleteBranches(ctx, apr, dbData)
 	default:
-		// regular branch - create new branch
-		if apr.NArg() != 1 {
-			return 1, InvalidArgErr
-		}
-
-		branchName := apr.Arg(0)
-		if len(branchName) == 0 {
-			return 1, EmptyBranchNameErr
-		}
-
-		err = createNewBranch(ctx, dbData, branchName)
-		if err != nil {
-			return 1, err
-		}
+		err = createNewBranch(ctx, dbData, apr)
 	}
 
-	return 0, nil
-}
-
-func createNewBranch(ctx *sql.Context, dbData env.DbData, branchName string) error {
-	// Check if the branch already exists.
-	isBranch, err := actions.IsBranch(ctx, dbData.Ddb, branchName)
 	if err != nil {
-		return err
-	} else if isBranch {
-		return errors.New(fmt.Sprintf("fatal: A branch named '%s' already exists.", branchName))
+		return 1, err
+	} else {
+		return 0, nil
 	}
-
-	startPt := fmt.Sprintf("head")
-	return actions.CreateBranchWithStartPt(ctx, dbData, branchName, startPt, false)
 }
 
-func makeACopyOfBranch(ctx *sql.Context, dbData env.DbData, apr *argparser.ArgParseResults) error {
+func renameBranch(ctx *sql.Context, dbData env.DbData, apr *argparser.ArgParseResults) error {
+	if apr.NArg() != 2 {
+		return InvalidArgErr
+	}
+	oldBranchName, newBranchName := apr.Arg(0), apr.Arg(1)
+	if oldBranchName == "" || newBranchName == "" {
+		return EmptyBranchNameErr
+	}
+	force := apr.Contains(cli.ForceFlag)
+
+	if !force {
+		err := validateBranchNotActiveInAnySession(ctx, oldBranchName)
+		if err != nil {
+			return err
+		}
+	}
+
+	return actions.RenameBranch(ctx, dbData, loadConfig(ctx), oldBranchName, newBranchName, force)
+}
+
+func deleteBranches(ctx *sql.Context, apr *argparser.ArgParseResults, dbData env.DbData) error {
+	if apr.NArg() == 0 {
+		return InvalidArgErr
+	}
+
+	for _, branchName := range apr.Args {
+		if len(branchName) == 0 {
+			return EmptyBranchNameErr
+		}
+		force := apr.Contains(cli.DeleteForceFlag) || apr.Contains(cli.ForceFlag)
+
+		if !force {
+			err := validateBranchNotActiveInAnySession(ctx, branchName)
+			if err != nil {
+				return err
+			}
+		}
+		err := actions.DeleteBranch(ctx, dbData, loadConfig(ctx), branchName, actions.DeleteOptions{
+			Force: force,
+		})
+		if err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
+// validateBranchNotActiveInAnySessions returns an error if the specified branch is currently
+// selected as the active branch for any active server sessions.
+func validateBranchNotActiveInAnySession(ctx *sql.Context, branchName string) error {
+	dbName := ctx.GetCurrentDatabase()
+	if dbName == "" {
+		return nil
+	}
+
+	if sqlserver.RunningInServerMode() == false {
+		return nil
+	}
+
+	runningServer := sqlserver.GetRunningServer()
+	if runningServer == nil {
+		return nil
+	}
+	sessionManager := runningServer.SessionManager()
+	branchRef := ref.NewBranchRef(branchName)
+
+	return sessionManager.Iter(func(session sql.Session) (bool, error) {
+		dsess, ok := session.(*dsess.DoltSession)
+		if !ok {
+			return false, fmt.Errorf("unexpected session type: %T", session)
+		}
+
+		activeBranchRef, err := dsess.CWBHeadRef(ctx, dbName)
+		if err != nil {
+			return false, err
+		}
+
+		if ref.Equals(branchRef, activeBranchRef) {
+			return false, fmt.Errorf("unsafe to delete or rename branches in use in other sessions; " +
+				"use --force to force the change")
+		}
+
+		return false, nil
+	})
+}
+
+func loadConfig(ctx *sql.Context) *env.DoltCliConfig {
+	// When executing branch actions from SQL, we don't have access to a DoltEnv like we do from
+	// within the CLI. We can fake it here enough to get a DoltCliConfig, but we can't rely on the
+	// DoltEnv because tests and production will run with different settings (e.g. in-mem versus file).
+	dEnv := env.Load(ctx, env.GetCurrentUserHomeDir, filesys.LocalFS, doltdb.LocalDirDoltDB, "")
+	return dEnv.Config
+}
+
+func createNewBranch(ctx *sql.Context, dbData env.DbData, apr *argparser.ArgParseResults) error {
+	if apr.NArg() != 1 {
+		return InvalidArgErr
+	}
+
+	branchName := apr.Arg(0)
+	if len(branchName) == 0 {
+		return EmptyBranchNameErr
+	}
+
+	return actions.CreateBranchWithStartPt(ctx, dbData, branchName, "HEAD", apr.Contains(cli.ForceFlag))
+}
+
+func copyBranch(ctx *sql.Context, dbData env.DbData, apr *argparser.ArgParseResults) error {
 	if apr.NArg() != 2 {
 		return InvalidArgErr
 	}
