@@ -24,8 +24,8 @@ var DoltProcedures = []sql.ExternalStoredProcedureDetails{
 	{Name: "dolt_clean", Schema: int64Schema("status"), Function: doltClean},
 	{Name: "dolt_commit", Schema: stringSchema("hash"), Function: doltCommit},
 	{Name: "dolt_fetch", Schema: int64Schema("success"), Function: doltFetch},
-	{Name: "dolt_merge", Schema: int64Schema("no_conflicts"), Function: doltMerge},
-	{Name: "dolt_pull", Schema: int64Schema("no_conflicts"), Function: doltPull},
+	{Name: "dolt_merge", Schema: int64Schema("fast_forward", "conflicts"), Function: doltMerge},
+	{Name: "dolt_pull", Schema: int64Schema("fast_forward", "conflicts"), Function: doltPull},
 	{Name: "dolt_push", Schema: int64Schema("success"), Function: doltPush},
 	{Name: "dolt_reset", Schema: int64Schema("status"), Function: doltReset},
 	{Name: "dolt_revert", Schema: int64Schema("status"), Function: doltRevert},
@@ -37,8 +37,8 @@ var DoltProcedures = []sql.ExternalStoredProcedureDetails{
 	{Name: "dclean", Schema: int64Schema("status"), Function: doltClean},
 	{Name: "dcommit", Schema: stringSchema("hash"), Function: doltCommit},
 	{Name: "dfetch", Schema: int64Schema("success"), Function: doltFetch},
-	{Name: "dmerge", Schema: int64Schema("no_conflicts"), Function: doltMerge},
-	{Name: "dpull", Schema: int64Schema("no_conflicts"), Function: doltPull},
+	{Name: "dmerge", Schema: int64Schema("fast_forward", "conflicts"), Function: doltMerge},
+	{Name: "dpull", Schema: int64Schema("fast_forward", "conflicts"), Function: doltPull},
 	{Name: "dpush", Schema: int64Schema("success"), Function: doltPush},
 	{Name: "dreset", Schema: int64Schema("status"), Function: doltReset},
 	{Name: "drevert", Schema: int64Schema("status"), Function: doltRevert},
