@@ -15,18 +15,24 @@
 package dtables
 
 import (
+	"bytes"
 	"context"
-	"errors"
+	"fmt"
 	"io"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/diff"
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
-	"github.com/dolthub/dolt/go/libraries/doltcore/env/actions"
-	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/index"
+	"github.com/dolthub/dolt/go/libraries/doltcore/env/actions/commitwalk"
+	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dsess"
+	"github.com/dolthub/dolt/go/store/datas"
+	"github.com/dolthub/dolt/go/store/hash"
 
 	"github.com/dolthub/go-mysql-server/sql"
 )
 
+var workingSetPartitionKey = []byte("workingset")
+var commitHistoryPartitionKey = []byte("commithistory")
+
 // UnscopedDiffTable is a sql.Table implementation of a system table that shows which tables have
 // changed in each commit, across all branches.
 type UnscopedDiffTable struct {
@@ -73,115 +79,197 @@ func (dt *UnscopedDiffTable) Schema() sql.Schema {
 	}
 }
 
-// Partitions is a sql.Table interface function that returns a partition of the data. Currently data is unpartitioned.
-func (dt *UnscopedDiffTable) Partitions(*sql.Context) (sql.PartitionIter, error) {
-	return index.SinglePartitionIterFromNomsMap(nil), nil
+// Partitions is a sql.Table interface function that returns a partition of the data. Returns one
+// partition for working set changes and one partition for all commit history.
+func (dt *UnscopedDiffTable) Partitions(ctx *sql.Context) (sql.PartitionIter, error) {
+	return NewSliceOfPartitionsItr([]sql.Partition{
+		newDoltDiffPartition(workingSetPartitionKey),
+		newDoltDiffPartition(commitHistoryPartitionKey),
+	}), nil
 }
 
 // PartitionRows is a sql.Table interface function that gets a row iterator for a partition.
-func (dt *UnscopedDiffTable) PartitionRows(ctx *sql.Context, _ sql.Partition) (sql.RowIter, error) {
-	return NewUnscopedDiffTableItr(ctx, dt.ddb, dt.head)
+func (dt *UnscopedDiffTable) PartitionRows(ctx *sql.Context, partition sql.Partition) (sql.RowIter, error) {
+	if bytes.Equal(partition.Key(), workingSetPartitionKey) {
+		return dt.newWorkingSetRowItr(ctx)
+	} else if bytes.Equal(partition.Key(), commitHistoryPartitionKey) {
+		return dt.newCommitHistoryRowItr(ctx)
+	} else {
+		return nil, fmt.Errorf("unexpected partition: %v", partition)
+	}
 }
 
-// UnscopedDiffTableItr is a sql.RowItr implementation which iterates over each commit as if it's a row in the table.
-type UnscopedDiffTableItr struct {
-	ctx             *sql.Context
-	ddb             *doltdb.DoltDB
-	commits         []*doltdb.Commit
-	commitIdx       int
-	tableChanges    []tableChange
-	tableChangesIdx int
-}
-
-// NewUnscopedDiffTableItr creates a UnscopedDiffTableItr from the current environment.
-func NewUnscopedDiffTableItr(ctx *sql.Context, ddb *doltdb.DoltDB, head *doltdb.Commit) (*UnscopedDiffTableItr, error) {
-	commits, err := actions.TimeSortedCommits(ctx, ddb, head, -1)
+func (dt *UnscopedDiffTable) newWorkingSetRowItr(ctx *sql.Context) (sql.RowIter, error) {
+	sess := dsess.DSessFromSess(ctx.Session)
+	roots, ok := sess.GetRoots(ctx, ctx.GetCurrentDatabase())
+	if !ok {
+		return nil, fmt.Errorf("unable to lookup roots for database %s", ctx.GetCurrentDatabase())
+	}
 
+	staged, unstaged, err := diff.GetStagedUnstagedTableDeltas(ctx, roots)
 	if err != nil {
 		return nil, err
 	}
 
-	return &UnscopedDiffTableItr{ctx, ddb, commits, 0, nil, -1}, nil
+	return &doltDiffWorkingSetRowItr{
+		tableDeltas: append(staged, unstaged[0:]...),
+	}, nil
 }
 
-// HasNext returns true if this UnscopedDiffItr has more elements left.
-func (itr *UnscopedDiffTableItr) HasNext() bool {
-	// There are more diff records to iterate over if:
-	//   1) there is more than one commit left to process, or
-	//   2) the tableChanges array isn't nilled out and has data left to process
-	return itr.commitIdx+1 < len(itr.commits) || itr.tableChanges != nil
+var _ sql.RowIter = &doltDiffWorkingSetRowItr{}
+
+type doltDiffWorkingSetRowItr struct {
+	tableDeltas []diff.TableDelta
+	idx         int
+}
+
+func (d *doltDiffWorkingSetRowItr) Next(ctx *sql.Context) (sql.Row, error) {
+	if d.idx >= len(d.tableDeltas) {
+		return nil, io.EOF
+	}
+
+	tableDelta := d.tableDeltas[d.idx]
+	d.idx++
+
+	change, err := processTableDelta(ctx, tableDelta)
+	if err != nil {
+		return nil, err
+	}
+
+	sqlRow := sql.NewRow(
+		"WORKING",
+		change.tableName,
+		"NULL", // committer
+		"NULL", // email
+		"NULL", // date
+		"NULL", // message
+		change.dataChange,
+		change.schemaChange,
+	)
+
+	return sqlRow, nil
+}
+
+func (d *doltDiffWorkingSetRowItr) Close(c *sql.Context) error {
+	return nil
+}
+
+var _ sql.Partition = &doltDiffPartition{}
+
+type doltDiffPartition struct {
+	key []byte
+}
+
+func newDoltDiffPartition(key []byte) *doltDiffPartition {
+	return &doltDiffPartition{
+		key: key,
+	}
+}
+
+func (d doltDiffPartition) Key() []byte {
+	return d.key
+}
+
+// doltDiffCommitHistoryRowItr is a sql.RowItr implementation which iterates over each commit as if it's a row in the table.
+type doltDiffCommitHistoryRowItr struct {
+	ctx             *sql.Context
+	ddb             *doltdb.DoltDB
+	child           doltdb.CommitItr
+	meta            *datas.CommitMeta
+	hash            hash.Hash
+	tableChanges    []tableChange
+	tableChangesIdx int
+}
+
+// newCommitHistoryRowItr creates a doltDiffCommitHistoryRowItr from the current environment.
+func (dt *UnscopedDiffTable) newCommitHistoryRowItr(ctx *sql.Context) (*doltDiffCommitHistoryRowItr, error) {
+	hash, err := dt.head.HashOf()
+	if err != nil {
+		return nil, err
+	}
+	child, err := commitwalk.GetTopologicalOrderIterator(ctx, dt.ddb, hash)
+
+	return &doltDiffCommitHistoryRowItr{
+		ctx:             ctx,
+		ddb:             dt.ddb,
+		child:           child,
+		tableChangesIdx: -1,
+	}, nil
 }
 
 // incrementIndexes increments the table changes index, and if it's the end of the table changes array, moves
 // to the next commit, and resets the table changes index so that it can be populated when Next() is called.
-func (itr *UnscopedDiffTableItr) incrementIndexes() {
+func (itr *doltDiffCommitHistoryRowItr) incrementIndexes() {
 	itr.tableChangesIdx++
 	if itr.tableChangesIdx >= len(itr.tableChanges) {
 		itr.tableChangesIdx = -1
 		itr.tableChanges = nil
-		itr.commitIdx++
 	}
 }
 
 // Next retrieves the next row. It will return io.EOF if it's the last row.
 // After retrieving the last row, Close will be automatically closed.
-func (itr *UnscopedDiffTableItr) Next(ctx *sql.Context) (sql.Row, error) {
-	if !itr.HasNext() {
-		return nil, io.EOF
-	}
+func (itr *doltDiffCommitHistoryRowItr) Next(ctx *sql.Context) (sql.Row, error) {
 	defer itr.incrementIndexes()
 
-	// Load table changes if we don't have them for this commit yet
 	for itr.tableChanges == nil {
-		err := itr.loadTableChanges(ctx, itr.commits[itr.commitIdx])
+		err := itr.loadTableChanges(ctx)
 		if err != nil {
 			return nil, err
 		}
 	}
 
-	commit := itr.commits[itr.commitIdx]
-	hash, err := commit.HashOf()
-	if err != nil {
-		return nil, err
-	}
-
-	meta, err := commit.GetCommitMeta(ctx)
-	if err != nil {
-		return nil, err
-	}
-
 	tableChange := itr.tableChanges[itr.tableChangesIdx]
+	meta := itr.meta
+	h := itr.hash
 
-	return sql.NewRow(hash.String(), tableChange.tableName, meta.Name, meta.Email, meta.Time(),
-		meta.Description, tableChange.dataChange, tableChange.schemaChange), nil
+	return sql.NewRow(
+		h.String(),
+		tableChange.tableName,
+		meta.Name,
+		meta.Email,
+		meta.Time(),
+		meta.Description,
+		tableChange.dataChange,
+		tableChange.schemaChange,
+	), nil
 }
 
-// loadTableChanges loads the set of table changes for the current commit into this iterator, taking
-// care of advancing the iterator if that commit didn't mutate any tables and checking for EOF condition.
-func (itr *UnscopedDiffTableItr) loadTableChanges(ctx context.Context, commit *doltdb.Commit) error {
+// loadTableChanges loads the next commit's table changes and metadata
+// into the iterator.
+func (itr *doltDiffCommitHistoryRowItr) loadTableChanges(ctx context.Context) error {
+	hash, commit, err := itr.child.Next(ctx)
+	if err != nil {
+		return err
+	}
+
 	tableChanges, err := itr.calculateTableChanges(ctx, commit)
 	if err != nil {
 		return err
 	}
 
-	// If there are no table deltas for this commit (e.g. a "dolt doc" commit),
-	// advance to the next commit, checking for EOF condition.
+	itr.tableChanges = tableChanges
+	itr.tableChangesIdx = 0
 	if len(tableChanges) == 0 {
-		itr.commitIdx++
-		if !itr.HasNext() {
-			return io.EOF
-		}
-	} else {
-		itr.tableChanges = tableChanges
-		itr.tableChangesIdx = 0
+		return nil
 	}
 
+	meta, err := commit.GetCommitMeta(ctx)
+	if err != nil {
+		return err
+	}
+	itr.meta = meta
+	itr.hash = hash
 	return nil
 }
 
 // calculateTableChanges calculates the tables that changed in the specified commit, by comparing that
 // commit with its immediate ancestor commit.
-func (itr *UnscopedDiffTableItr) calculateTableChanges(ctx context.Context, commit *doltdb.Commit) ([]tableChange, error) {
+func (itr *doltDiffCommitHistoryRowItr) calculateTableChanges(ctx context.Context, commit *doltdb.Commit) ([]tableChange, error) {
+	if len(commit.DatasParents()) == 0 {
+		return nil, nil
+	}
+
 	toRootValue, err := commit.GetRootValue(ctx)
 	if err != nil {
 		return nil, err
@@ -204,7 +292,7 @@ func (itr *UnscopedDiffTableItr) calculateTableChanges(ctx context.Context, comm
 
 	tableChanges := make([]tableChange, len(deltas))
 	for i := 0; i < len(deltas); i++ {
-		change, err := itr.processTableDelta(deltas[i])
+		change, err := processTableDelta(itr.ctx, deltas[i])
 		if err != nil {
 			return nil, err
 		}
@@ -222,10 +310,10 @@ func (itr *UnscopedDiffTableItr) calculateTableChanges(ctx context.Context, comm
 
 // processTableDelta processes the specified TableDelta to determine what kind of change it was (i.e. table drop,
 // table rename, table create, or data update) and returns a tableChange struct representing the change.
-func (itr *UnscopedDiffTableItr) processTableDelta(delta diff.TableDelta) (*tableChange, error) {
+func processTableDelta(ctx *sql.Context, delta diff.TableDelta) (*tableChange, error) {
 	// Dropping a table is always a schema change, and also a data change if the table contained data
-	if itr.isTableDropChange(delta) {
-		isEmpty, err := itr.isTableDataEmpty(delta.FromTable)
+	if delta.IsDrop() {
+		isEmpty, err := isTableDataEmpty(ctx, delta.FromTable)
 		if err != nil {
 			return nil, err
 		}
@@ -238,8 +326,8 @@ func (itr *UnscopedDiffTableItr) processTableDelta(delta diff.TableDelta) (*tabl
 	}
 
 	// Renaming a table is always a schema change, and also a data change if the table data differs
-	if itr.isRenameChange(delta) {
-		dataChanged, err := itr.isTableDataDifferent(delta)
+	if delta.IsRename() {
+		dataChanged, err := delta.HasHashChanged()
 		if err != nil {
 			return nil, err
 		}
@@ -252,8 +340,8 @@ func (itr *UnscopedDiffTableItr) processTableDelta(delta diff.TableDelta) (*tabl
 	}
 
 	// Creating a table is always a schema change, and also a data change if data was inserted
-	if itr.isTableCreateChange(delta) {
-		isEmpty, err := itr.isTableDataEmpty(delta.ToTable)
+	if delta.IsAdd() {
+		isEmpty, err := isTableDataEmpty(ctx, delta.ToTable)
 		if err != nil {
 			return nil, err
 		}
@@ -265,12 +353,12 @@ func (itr *UnscopedDiffTableItr) processTableDelta(delta diff.TableDelta) (*tabl
 		}, nil
 	}
 
-	dataChanged, err := itr.isTableDataDifferent(delta)
+	dataChanged, err := delta.HasHashChanged()
 	if err != nil {
 		return nil, err
 	}
 
-	schemaChanged, err := itr.isTableSchemaDifferent(delta)
+	schemaChanged, err := delta.HasSchemaChanged(ctx)
 	if err != nil {
 		return nil, err
 	}
@@ -283,73 +371,16 @@ func (itr *UnscopedDiffTableItr) processTableDelta(delta diff.TableDelta) (*tabl
 }
 
 // Close closes the iterator.
-func (itr *UnscopedDiffTableItr) Close(*sql.Context) error {
+func (itr *doltDiffCommitHistoryRowItr) Close(*sql.Context) error {
 	return nil
 }
 
 // isTableDataEmpty return true if the table does not contain any data
-func (itr *UnscopedDiffTableItr) isTableDataEmpty(table *doltdb.Table) (bool, error) {
-	rowData, err := table.GetRowData(itr.ctx)
+func isTableDataEmpty(ctx *sql.Context, table *doltdb.Table) (bool, error) {
+	rowData, err := table.GetRowData(ctx)
 	if err != nil {
 		return false, err
 	}
 
 	return rowData.Empty(), nil
 }
-
-// isRenameChange returns true if the specified TableDelta represents a table rename change.
-func (itr *UnscopedDiffTableItr) isRenameChange(delta diff.TableDelta) bool {
-	return delta.FromTable != nil &&
-		delta.ToTable != nil &&
-		delta.FromName != delta.ToName
-}
-
-// isTableDropChange return true if the specified TableDelta represents a table drop change.
-func (itr *UnscopedDiffTableItr) isTableDropChange(delta diff.TableDelta) bool {
-	return len(delta.FromName) > 0 && len(delta.ToName) == 0
-}
-
-// isTableCreateChange returns true if the specified TableDelta represents a table create change.
-func (itr *UnscopedDiffTableItr) isTableCreateChange(delta diff.TableDelta) bool {
-	return delta.FromTable == nil && delta.ToTable != nil
-}
-
-// isTableDataDifferent returns true if the data in the from and to tables is different. This method
-// should only be called with both from and to tables are not nil.
-func (itr *UnscopedDiffTableItr) isTableDataDifferent(delta diff.TableDelta) (bool, error) {
-	if delta.FromTable == nil || delta.ToTable == nil {
-		return false, errors.New("specified FromTable and ToTable should never be nil")
-	}
-
-	fromTableHash, err := delta.FromTable.HashOf()
-	if err != nil {
-		return false, err
-	}
-
-	toTableHash, err := delta.ToTable.HashOf()
-	if err != nil {
-		return false, err
-	}
-
-	return fromTableHash != toTableHash, nil
-}
-
-// isTableSchemaDifferent returns true if the schema in the from and to tables is different. This method
-// should only be called with both from and to tables are not nil.
-func (itr *UnscopedDiffTableItr) isTableSchemaDifferent(delta diff.TableDelta) (bool, error) {
-	if delta.FromTable == nil || delta.ToTable == nil {
-		return false, errors.New("specified FromTable and ToTable should never be nil")
-	}
-
-	fromSchemaHash, err := delta.FromTable.GetSchemaHash(itr.ctx)
-	if err != nil {
-		return false, err
-	}
-
-	toSchemaHash, err := delta.ToTable.GetSchemaHash(itr.ctx)
-	if err != nil {
-		return false, err
-	}
-
-	return fromSchemaHash != toSchemaHash, nil
-}
