@@ -19,10 +19,13 @@ import (
 	"testing"
 
 	"github.com/dolthub/go-mysql-server/enginetest"
+	"github.com/dolthub/go-mysql-server/enginetest/queries"
+	"github.com/dolthub/go-mysql-server/enginetest/scriptgen/setup"
 	"github.com/dolthub/go-mysql-server/sql"
 	"github.com/stretchr/testify/require"
 
 	"github.com/dolthub/dolt/go/libraries/doltcore/doltdb"
+	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/dsess"
 	"github.com/dolthub/dolt/go/store/types"
 )
 
@@ -36,13 +39,14 @@ func TestDoltTransactionCommitOneClient(t *testing.T) {
 	// In this test, we're setting only one client to match transaction commits to dolt commits.
 	// Autocommit is disabled for the enabled client, as it's the recommended way to use this feature.
 	harness := newDoltHarness(t)
-	enginetest.TestTransactionScript(t, harness, enginetest.TransactionTest{
+	harness.Setup(setup.MydbData)
+	enginetest.TestTransactionScript(t, harness, queries.TransactionTest{
 		Name: "dolt commit on transaction commit one client",
 		SetUpScript: []string{
 			"CREATE TABLE x (y BIGINT PRIMARY KEY, z BIGINT);",
 			"INSERT INTO x VALUES (1,1);",
 		},
-		Assertions: []enginetest.ScriptTestAssertion{
+		Assertions: []queries.ScriptTestAssertion{
 			{
 				Query:    "/* client a */ SET @@autocommit=0;",
 				Expected: []sql.Row{{}},
@@ -146,8 +150,13 @@ func TestDoltTransactionCommitOneClient(t *testing.T) {
 			},
 		},
 	})
+	_, err := harness.NewEngine(t)
 
-	db := harness.databases[0].GetDoltDB()
+	ctx := enginetest.NewContext(harness)
+	db, ok := ctx.Session.(*dsess.DoltSession).GetDoltDB(ctx, "mydb")
+	if !ok {
+		t.Fatal("'mydb' database not found")
+	}
 	cs, err := doltdb.NewCommitSpec("HEAD")
 	require.NoError(t, err)
 	headRefs, err := db.GetHeadRefs(context.Background())
@@ -164,7 +173,7 @@ func TestDoltTransactionCommitOneClient(t *testing.T) {
 	require.NoError(t, err)
 	icm, err := initialCommit.GetCommitMeta(context.Background())
 	require.NoError(t, err)
-	require.Equal(t, "Initialize data repository", icm.Description)
+	require.Equal(t, "checkpoint enginetest database mydb", icm.Description)
 }
 
 func TestDoltTransactionCommitTwoClients(t *testing.T) {
@@ -175,13 +184,13 @@ func TestDoltTransactionCommitTwoClients(t *testing.T) {
 	// In this test, we're setting both clients to match transaction commits to dolt commits.
 	// Autocommit is disabled, as it's the recommended way to use this feature.
 	harness := newDoltHarness(t)
-	enginetest.TestTransactionScript(t, harness, enginetest.TransactionTest{
+	enginetest.TestTransactionScript(t, harness, queries.TransactionTest{
 		Name: "dolt commit on transaction commit two clients",
 		SetUpScript: []string{
 			"CREATE TABLE x (y BIGINT PRIMARY KEY, z BIGINT);",
 			"INSERT INTO x VALUES (1,1);",
 		},
-		Assertions: []enginetest.ScriptTestAssertion{
+		Assertions: []queries.ScriptTestAssertion{
 			{
 				Query:    "/* client a */ SET @@autocommit=0;",
 				Expected: []sql.Row{{}},
@@ -273,7 +282,13 @@ func TestDoltTransactionCommitTwoClients(t *testing.T) {
 			},
 		},
 	})
-	db := harness.databases[0].GetDoltDB()
+	_, err := harness.NewEngine(t)
+
+	ctx := enginetest.NewContext(harness)
+	db, ok := ctx.Session.(*dsess.DoltSession).GetDoltDB(ctx, "mydb")
+	if !ok {
+		t.Fatal("'mydb' database not found")
+	}
 	cs, err := doltdb.NewCommitSpec("HEAD")
 	require.NoError(t, err)
 	headRefs, err := db.GetHeadRefs(context.Background())
@@ -296,7 +311,7 @@ func TestDoltTransactionCommitTwoClients(t *testing.T) {
 	require.NoError(t, err)
 	cm0, err := commit0.GetCommitMeta(context.Background())
 	require.NoError(t, err)
-	require.Equal(t, "Initialize data repository", cm0.Description)
+	require.Equal(t, "checkpoint enginetest database mydb", cm0.Description)
 }
 
 func TestDoltTransactionCommitAutocommit(t *testing.T) {
@@ -307,13 +322,13 @@ func TestDoltTransactionCommitAutocommit(t *testing.T) {
 	// In this test, each insertion from both clients cause a commit as autocommit is enabled.
 	// Not the recommended way to use the feature, but it's permitted.
 	harness := newDoltHarness(t)
-	enginetest.TestTransactionScript(t, harness, enginetest.TransactionTest{
+	enginetest.TestTransactionScript(t, harness, queries.TransactionTest{
 		Name: "dolt commit with autocommit",
 		SetUpScript: []string{
 			"CREATE TABLE x (y BIGINT PRIMARY KEY, z BIGINT);",
 			"INSERT INTO x VALUES (1,1);",
 		},
-		Assertions: []enginetest.ScriptTestAssertion{
+		Assertions: []queries.ScriptTestAssertion{
 			// these SET statements currently commit a transaction (since autocommit is on)
 			{
 				Query:    "/* client a */ SET @@dolt_transaction_commit=1;",
@@ -345,7 +360,13 @@ func TestDoltTransactionCommitAutocommit(t *testing.T) {
 			},
 		},
 	})
-	db := harness.databases[0].GetDoltDB()
+	_, err := harness.NewEngine(t)
+
+	ctx := enginetest.NewContext(harness)
+	db, ok := ctx.Session.(*dsess.DoltSession).GetDoltDB(ctx, "mydb")
+	if !ok {
+		t.Fatal("'mydb' database not found")
+	}
 	cs, err := doltdb.NewCommitSpec("HEAD")
 	require.NoError(t, err)
 	headRefs, err := db.GetHeadRefs(context.Background())
@@ -374,7 +395,7 @@ func TestDoltTransactionCommitAutocommit(t *testing.T) {
 	require.NoError(t, err)
 	cm0, err := commit0.GetCommitMeta(context.Background())
 	require.NoError(t, err)
-	require.Equal(t, "Initialize data repository", cm0.Description)
+	require.Equal(t, "checkpoint enginetest database mydb", cm0.Description)
 }
 
 func TestDoltTransactionCommitLateFkResolution(t *testing.T) {
@@ -383,7 +404,7 @@ func TestDoltTransactionCommitLateFkResolution(t *testing.T) {
 	}
 
 	harness := newDoltHarness(t)
-	enginetest.TestTransactionScript(t, harness, enginetest.TransactionTest{
+	enginetest.TestTransactionScript(t, harness, queries.TransactionTest{
 		Name: "delayed foreign key resolution with transaction commits",
 		SetUpScript: []string{
 			"SET foreign_key_checks=0;",
@@ -392,7 +413,7 @@ func TestDoltTransactionCommitLateFkResolution(t *testing.T) {
 			"CREATE TABLE parent (pk BIGINT PRIMARY KEY);",
 			"INSERT INTO parent VALUES (1), (2);",
 		},
-		Assertions: []enginetest.ScriptTestAssertion{
+		Assertions: []queries.ScriptTestAssertion{
 			{
 				Query:    "/* client a */ SET @@autocommit=0;",
 				Expected: []sql.Row{{}},
