@@ -16,15 +16,70 @@ package index
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
+	"math"
 	"time"
 
 	"github.com/dolthub/go-mysql-server/sql"
+	"github.com/shopspring/decimal"
 
+	"github.com/dolthub/dolt/go/libraries/doltcore/schema/typeinfo"
 	geo "github.com/dolthub/dolt/go/store/geometry"
+	"github.com/dolthub/dolt/go/store/types"
 	"github.com/dolthub/dolt/go/store/val"
 )
 
+var ErrValueExceededMaxFieldSize = errors.New("value exceeded max field size of 65kb")
+
+// todo(andy): this should go in GMS
+func DenormalizeRow(sch sql.Schema, row sql.Row) (sql.Row, error) {
+	var err error
+	for i := range row {
+		if row[i] == nil {
+			continue
+		}
+		switch typ := sch[i].Type.(type) {
+		case sql.DecimalType:
+			row[i] = row[i].(decimal.Decimal).String()
+		case sql.EnumType:
+			row[i], err = typ.Unmarshal(int64(row[i].(uint16)))
+		case sql.SetType:
+			row[i], err = typ.Unmarshal(row[i].(uint64))
+		default:
+		}
+		if err != nil {
+			return nil, err
+		}
+	}
+	return row, nil
+}
+
+// todo(andy): this should go in GMS
+func NormalizeRow(sch sql.Schema, row sql.Row) (sql.Row, error) {
+	var err error
+	for i := range row {
+		if row[i] == nil {
+			continue
+		}
+		switch typ := sch[i].Type.(type) {
+		case sql.DecimalType:
+			row[i], err = decimal.NewFromString(row[i].(string))
+		case sql.EnumType:
+			var v int64
+			v, err = typ.Marshal(row[i])
+			row[i] = uint16(v)
+		case sql.SetType:
+			row[i], err = typ.Marshal(row[i])
+		default:
+		}
+		if err != nil {
+			return nil, err
+		}
+	}
+	return row, nil
+}
+
 // GetField reads the value from the ith field of the Tuple as an interface{}.
 func GetField(td val.TupleDesc, i int, tup val.Tuple) (v interface{}, err error) {
 	var ok bool
@@ -49,14 +104,26 @@ func GetField(td val.TupleDesc, i int, tup val.Tuple) (v interface{}, err error)
 		v, ok = td.GetFloat32(i, tup)
 	case val.Float64Enc:
 		v, ok = td.GetFloat64(i, tup)
+	case val.Bit64Enc:
+		v, ok = td.GetBit(i, tup)
 	case val.DecimalEnc:
 		v, ok = td.GetDecimal(i, tup)
-	case val.TimeEnc:
-		v, ok = td.GetSqlTime(i, tup)
 	case val.YearEnc:
 		v, ok = td.GetYear(i, tup)
-	case val.TimestampEnc, val.DateEnc, val.DatetimeEnc:
-		v, ok = td.GetTimestamp(i, tup)
+	case val.DateEnc:
+		v, ok = td.GetDate(i, tup)
+	case val.TimeEnc:
+		var t int64
+		t, ok = td.GetSqlTime(i, tup)
+		if ok {
+			v, err = deserializeTime(t)
+		}
+	case val.DatetimeEnc:
+		v, ok = td.GetDatetime(i, tup)
+	case val.EnumEnc:
+		v, ok = td.GetEnum(i, tup)
+	case val.SetEnc:
+		v, ok = td.GetSet(i, tup)
 	case val.StringEnc:
 		v, ok = td.GetString(i, tup)
 	case val.ByteStringEnc:
@@ -75,6 +142,10 @@ func GetField(td val.TupleDesc, i int, tup val.Tuple) (v interface{}, err error)
 		if ok {
 			v = deserializeGeometry(buf)
 		}
+	case val.Hash128Enc:
+		v, ok = td.GetHash128(i, tup)
+	case val.AddressEnc:
+		v, ok = td.GetAddress(i, tup)
 	default:
 		panic("unknown val.encoding")
 	}
@@ -112,29 +183,55 @@ func PutField(tb *val.TupleBuilder, i int, v interface{}) error {
 		tb.PutFloat32(i, v.(float32))
 	case val.Float64Enc:
 		tb.PutFloat64(i, v.(float64))
+	case val.Bit64Enc:
+		tb.PutBit(i, uint64(convUint(v)))
 	case val.DecimalEnc:
-		tb.PutDecimal(i, v.(string))
-	case val.TimeEnc:
-		tb.PutSqlTime(i, v.(string))
+		tb.PutDecimal(i, v.(decimal.Decimal))
 	case val.YearEnc:
 		tb.PutYear(i, v.(int16))
-	case val.DateEnc, val.DatetimeEnc, val.TimestampEnc:
-		tb.PutTimestamp(i, v.(time.Time))
+	case val.DateEnc:
+		tb.PutDate(i, v.(time.Time))
+	case val.TimeEnc:
+		t, err := serializeTime(v)
+		if err != nil {
+			return err
+		}
+		tb.PutSqlTime(i, t)
+	case val.DatetimeEnc:
+		tb.PutDatetime(i, v.(time.Time))
+	case val.EnumEnc:
+		tb.PutEnum(i, v.(uint16))
+	case val.SetEnc:
+		tb.PutSet(i, v.(uint64))
 	case val.StringEnc:
 		tb.PutString(i, v.(string))
 	case val.ByteStringEnc:
 		if s, ok := v.(string); ok {
+			if len(s) > math.MaxUint16 {
+				return ErrValueExceededMaxFieldSize
+			}
 			v = []byte(s)
 		}
 		tb.PutByteString(i, v.([]byte))
 	case val.GeometryEnc:
+		geo := serializeGeometry(v)
+		if len(geo) > math.MaxUint16 {
+			return ErrValueExceededMaxFieldSize
+		}
 		tb.PutGeometry(i, serializeGeometry(v))
 	case val.JSONEnc:
 		buf, err := convJson(v)
+		if len(buf) > math.MaxUint16 {
+			return ErrValueExceededMaxFieldSize
+		}
 		if err != nil {
 			return err
 		}
 		tb.PutJSON(i, buf)
+	case val.Hash128Enc:
+		tb.PutHash128(i, v.([]byte))
+	case val.AddressEnc:
+		tb.PutAddress(i, v.([]byte))
 	default:
 		panic(fmt.Sprintf("unknown encoding %v %v", enc, v))
 	}
@@ -193,22 +290,14 @@ func convUint(v interface{}) uint {
 	}
 }
 
-func convJson(v interface{}) (buf []byte, err error) {
-	v, err = sql.JSON.Convert(v)
-	if err != nil {
-		return nil, err
-	}
-	return json.Marshal(v.(sql.JSONDocument).Val)
-}
-
 func deserializeGeometry(buf []byte) (v interface{}) {
 	srid, _, typ := geo.ParseEWKBHeader(buf)
 	buf = buf[geo.EWKBHeaderSize:]
 	switch typ {
 	case geo.PointType:
 		v = geo.DeserializePoint(buf, srid)
-	case geo.LinestringType:
-		v = geo.DeserializeLinestring(buf, srid)
+	case geo.LineStringType:
+		v = geo.DeserializeLineString(buf, srid)
 	case geo.PolygonType:
 		v = geo.DeserializePolygon(srid, buf)
 	default:
@@ -221,11 +310,31 @@ func serializeGeometry(v interface{}) []byte {
 	switch t := v.(type) {
 	case sql.Point:
 		return geo.SerializePoint(t)
-	case sql.Linestring:
-		return geo.SerializeLinestring(t)
+	case sql.LineString:
+		return geo.SerializeLineString(t)
 	case sql.Polygon:
 		return geo.SerializePolygon(t)
 	default:
 		panic(fmt.Sprintf("unknown geometry %v", v))
 	}
 }
+
+func convJson(v interface{}) (buf []byte, err error) {
+	v, err = sql.JSON.Convert(v)
+	if err != nil {
+		return nil, err
+	}
+	return json.Marshal(v.(sql.JSONDocument).Val)
+}
+
+func deserializeTime(v int64) (interface{}, error) {
+	return typeinfo.TimeType.ConvertNomsValueToValue(types.Int(v))
+}
+
+func serializeTime(v interface{}) (int64, error) {
+	i, err := typeinfo.TimeType.ConvertValueToNomsValue(nil, nil, v)
+	if err != nil {
+		return 0, err
+	}
+	return int64(i.(types.Int)), nil
+}
