@@ -22,6 +22,7 @@ import (
 
 	"github.com/dolthub/go-mysql-server/sql"
 	"github.com/dolthub/go-mysql-server/sql/expression/function"
+	"github.com/shopspring/decimal"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
@@ -95,6 +96,16 @@ func TestRoundTripProllyFields(t *testing.T) {
 			typ:   val.Type{Enc: val.Float64Enc},
 			value: float64(-math.Pi),
 		},
+		{
+			name:  "bit",
+			typ:   val.Type{Enc: val.Bit64Enc},
+			value: uint64(42),
+		},
+		{
+			name:  "decimal",
+			typ:   val.Type{Enc: val.DecimalEnc},
+			value: mustParseDecimal("0.263419374632932747932030573792"),
+		},
 		{
 			name:  "string",
 			typ:   val.Type{Enc: val.StringEnc},
@@ -113,17 +124,22 @@ func TestRoundTripProllyFields(t *testing.T) {
 		{
 			name:  "date",
 			typ:   val.Type{Enc: val.DateEnc},
-			value: time.Now().UTC(),
+			value: dateFromTime(time.Now().UTC()),
+		},
+		{
+			name:  "time",
+			typ:   val.Type{Enc: val.TimeEnc},
+			value: "11:22:00",
 		},
 		{
 			name:  "datetime",
 			typ:   val.Type{Enc: val.DatetimeEnc},
-			value: time.Now().UTC(),
+			value: time.UnixMicro(time.Now().UTC().UnixMicro()).UTC(),
 		},
 		{
 			name:  "timestamp",
-			typ:   val.Type{Enc: val.TimestampEnc},
-			value: time.Now().UTC(),
+			typ:   val.Type{Enc: val.DatetimeEnc},
+			value: time.UnixMicro(time.Now().UTC().UnixMicro()).UTC(),
 		},
 		{
 			name:  "json",
@@ -196,3 +212,16 @@ func mustParseJson(t *testing.T, s string) sql.JSONDocument {
 	require.NoError(t, err)
 	return sql.JSONDocument{Val: v}
 }
+
+func mustParseDecimal(s string) decimal.Decimal {
+	d, err := decimal.NewFromString(s)
+	if err != nil {
+		panic(err)
+	}
+	return d
+}
+
+func dateFromTime(t time.Time) time.Time {
+	y, m, d := t.Year(), t.Month(), t.Day()
+	return time.Date(y, m, d, 0, 0, 0, 0, time.UTC)
+}
