@@ -279,7 +279,12 @@ func fragFromSchemasTable(ctx *sql.Context, tbl *WritableDoltTable, fragType str
 		return nil, false, err
 	}
 
-	iter, err := index.RowIterForIndexLookup(ctx, lookup, tbl.sqlSch, nil)
+	dt, err := tbl.doltTable(ctx)
+	if err != nil {
+		return nil, false, err
+	}
+
+	iter, err := index.RowIterForIndexLookup(ctx, dt, lookup, tbl.sqlSch, nil)
 	if err != nil {
 		return nil, false, err
 	}
