@@ -92,24 +92,32 @@ func (db *SingleTableInfoDatabase) PartitionRows2(ctx *sql.Context, part sql.Par
 	return nil, fmt.Errorf("cannot get partition rows of a single table information database")
 }
 
-// GetForeignKeys implements sql.ForeignKeyTable.
-func (db *SingleTableInfoDatabase) GetForeignKeys(ctx *sql.Context) ([]sql.ForeignKeyConstraint, error) {
+// CreateIndexForForeignKey implements sql.ForeignKeyTable.
+func (db *SingleTableInfoDatabase) CreateIndexForForeignKey(ctx *sql.Context, indexName string, using sql.IndexUsing, constraint sql.IndexConstraint, columns []sql.IndexColumn) error {
+	return fmt.Errorf("cannot create foreign keys on a single table information database")
+}
+
+// GetDeclaredForeignKeys implements sql.ForeignKeyTable.
+func (db *SingleTableInfoDatabase) GetDeclaredForeignKeys(ctx *sql.Context) ([]sql.ForeignKeyConstraint, error) {
 	fks := make([]sql.ForeignKeyConstraint, len(db.foreignKeys))
 	for i, fk := range db.foreignKeys {
 		if !fk.IsResolved() {
 			fks[i] = sql.ForeignKeyConstraint{
-				Name:              fk.Name,
-				Columns:           fk.UnresolvedFKDetails.TableColumns,
-				ReferencedTable:   fk.ReferencedTableName,
-				ReferencedColumns: fk.UnresolvedFKDetails.ReferencedTableColumns,
-				OnUpdate:          toReferenceOption(fk.OnUpdate),
-				OnDelete:          toReferenceOption(fk.OnDelete),
+				Name:           fk.Name,
+				Database:       ctx.GetCurrentDatabase(),
+				Table:          fk.TableName,
+				Columns:        fk.UnresolvedFKDetails.TableColumns,
+				ParentDatabase: ctx.GetCurrentDatabase(),
+				ParentTable:    fk.ReferencedTableName,
+				ParentColumns:  fk.UnresolvedFKDetails.ReferencedTableColumns,
+				OnUpdate:       toReferentialAction(fk.OnUpdate),
+				OnDelete:       toReferentialAction(fk.OnDelete),
 			}
 			continue
 		}
 		if parentSch, ok := db.parentSchs[fk.ReferencedTableName]; ok {
 			var err error
-			fks[i], err = toForeignKeyConstraint(fk, db.sch, parentSch)
+			fks[i], err = toForeignKeyConstraint(fk, ctx.GetCurrentDatabase(), db.sch, parentSch)
 			if err != nil {
 				return nil, err
 			}
@@ -121,6 +129,31 @@ func (db *SingleTableInfoDatabase) GetForeignKeys(ctx *sql.Context) ([]sql.Forei
 	return fks, nil
 }
 
+// GetReferencedForeignKeys implements sql.ForeignKeyTable.
+func (db *SingleTableInfoDatabase) GetReferencedForeignKeys(ctx *sql.Context) ([]sql.ForeignKeyConstraint, error) {
+	return nil, nil
+}
+
+// AddForeignKey implements sql.ForeignKeyTable.
+func (db *SingleTableInfoDatabase) AddForeignKey(ctx *sql.Context, fk sql.ForeignKeyConstraint) error {
+	return fmt.Errorf("cannot create foreign keys on a single table information database")
+}
+
+// DropForeignKey implements sql.ForeignKeyTable.
+func (db *SingleTableInfoDatabase) DropForeignKey(ctx *sql.Context, fkName string) error {
+	return fmt.Errorf("cannot create foreign keys on a single table information database")
+}
+
+// UpdateForeignKey implements sql.ForeignKeyTable.
+func (db *SingleTableInfoDatabase) UpdateForeignKey(ctx *sql.Context, fkName string, fk sql.ForeignKeyConstraint) error {
+	return fmt.Errorf("cannot create foreign keys on a single table information database")
+}
+
+// GetForeignKeyUpdater implements sql.ForeignKeyTable.
+func (db *SingleTableInfoDatabase) GetForeignKeyUpdater(ctx *sql.Context) sql.ForeignKeyUpdater {
+	return nil
+}
+
 // WithIndexLookup implements sql.IndexedTable.
 func (db *SingleTableInfoDatabase) WithIndexLookup(sql.IndexLookup) sql.Table {
 	return db
@@ -184,22 +217,22 @@ type fmtIndex struct {
 	comment   string
 }
 
-// ID implementes sql.Index
+// ID implements sql.Index
 func (idx fmtIndex) ID() string {
 	return idx.id
 }
 
-// Database implementes sql.Index
+// Database implements sql.Index
 func (idx fmtIndex) Database() string {
 	return idx.db
 }
 
-// Table implementes sql.Index
+// Table implements sql.Index
 func (idx fmtIndex) Table() string {
 	return idx.tbl
 }
 
-// Expressions implementes sql.Index
+// Expressions implements sql.Index
 func (idx fmtIndex) Expressions() []string {
 	strs := make([]string, len(idx.cols))
 	for i, col := range idx.cols {
@@ -208,32 +241,32 @@ func (idx fmtIndex) Expressions() []string {
 	return strs
 }
 
-// IsUnique implementes sql.Index
+// IsUnique implements sql.Index
 func (idx fmtIndex) IsUnique() bool {
 	return idx.unique
 }
 
-// Comment implementes sql.Index
+// Comment implements sql.Index
 func (idx fmtIndex) Comment() string {
 	return idx.comment
 }
 
-// IndexType implementes sql.Index
+// IndexType implements sql.Index
 func (idx fmtIndex) IndexType() string {
 	return "BTREE"
 }
 
-// IsGenerated implementes sql.Index
+// IsGenerated implements sql.Index
 func (idx fmtIndex) IsGenerated() bool {
 	return idx.generated
 }
 
-// NewLookup implementes sql.Index
+// NewLookup implements sql.Index
 func (idx fmtIndex) NewLookup(ctx *sql.Context, ranges ...sql.Range) (sql.IndexLookup, error) {
 	panic("unimplemented")
 }
 
-// ColumnExpressionTypes implementes sql.Index
+// ColumnExpressionTypes implements sql.Index
 func (idx fmtIndex) ColumnExpressionTypes(ctx *sql.Context) []sql.ColumnExpressionType {
 	panic("unimplemented")
 }
