@@ -119,9 +119,44 @@ func NewDoltTable(name string, sch schema.Schema, tbl *doltdb.Table, db SqlDatab
 // LockedToRoot returns a version of this table with its root value locked to the given value. The table's values will
 // not change as the session's root value changes. Appropriate for AS OF queries, or other use cases where the table's
 // values should not change throughout execution of a session.
-func (t DoltTable) LockedToRoot(rootValue *doltdb.RootValue) *DoltTable {
-	t.lockedToRoot = rootValue
-	return &t
+func (t DoltTable) LockedToRoot(ctx *sql.Context, root *doltdb.RootValue) (*DoltTable, error) {
+	tbl, ok, err := root.GetTable(ctx, t.tableName)
+	if err != nil {
+		return nil, err
+	} else if !ok {
+		return nil, doltdb.ErrTableNotFound
+	}
+
+	sch, err := tbl.GetSchema(ctx)
+	if err != nil {
+		return nil, err
+	}
+
+	var autoCol schema.Column
+	_ = sch.GetAllCols().Iter(func(tag uint64, col schema.Column) (stop bool, err error) {
+		if col.AutoIncrement {
+			autoCol = col
+			stop = true
+		}
+		return
+	})
+
+	sqlSch, err := sqlutil.FromDoltSchema(t.tableName, sch)
+	if err != nil {
+		return nil, err
+	}
+
+	return &DoltTable{
+		tableName:     t.tableName,
+		db:            t.db,
+		nbf:           tbl.Format(),
+		sch:           sch,
+		sqlSch:        sqlSch,
+		autoIncCol:    autoCol,
+		projectedCols: t.projectedCols,
+		opts:          t.opts,
+		lockedToRoot:  root,
+	}, nil
 }
 
 // Internal interface for declaring the interfaces that read-only dolt tables are expected to implement
@@ -189,6 +224,16 @@ func (t *DoltTable) GetIndexes(ctx *sql.Context) ([]sql.Index, error) {
 	return index.DoltIndexesFromTable(ctx, t.db.Name(), t.tableName, tbl)
 }
 
+// HasIndex returns whether the given index is present in the table
+func (t *DoltTable) HasIndex(ctx *sql.Context, idx sql.Index) (bool, error) {
+	tbl, err := t.doltTable(ctx)
+	if err != nil {
+		return false, err
+	}
+
+	return index.TableHasIndex(ctx, t.db.Name(), t.tableName, tbl, idx)
+}
+
 // GetAutoIncrementValue gets the last AUTO_INCREMENT value
 func (t *DoltTable) GetAutoIncrementValue(ctx *sql.Context) (interface{}, error) {
 	table, err := t.doltTable(ctx)
@@ -325,7 +370,7 @@ func (t *DoltTable) PartitionRows(ctx *sql.Context, partition sql.Partition) (sq
 		return nil, err
 	}
 
-	return partitionRows(ctx, table, t.projectedCols, partition)
+	return partitionRows(ctx, table, t.sqlSch.Schema, t.projectedCols, partition)
 }
 
 func (t DoltTable) PartitionRows2(ctx *sql.Context, part sql.Partition) (sql.RowIter2, error) {
@@ -334,7 +379,7 @@ func (t DoltTable) PartitionRows2(ctx *sql.Context, part sql.Partition) (sql.Row
 		return nil, err
 	}
 
-	iter, err := partitionRows(ctx, table, t.projectedCols, part)
+	iter, err := partitionRows(ctx, table, t.sqlSch.Schema, t.projectedCols, part)
 	if err != nil {
 		return nil, err
 	}
@@ -342,12 +387,12 @@ func (t DoltTable) PartitionRows2(ctx *sql.Context, part sql.Partition) (sql.Row
 	return iter.(sql.RowIter2), err
 }
 
-func partitionRows(ctx *sql.Context, t *doltdb.Table, projCols []string, partition sql.Partition) (sql.RowIter, error) {
+func partitionRows(ctx *sql.Context, t *doltdb.Table, sqlSch sql.Schema, projCols []string, partition sql.Partition) (sql.RowIter, error) {
 	switch typedPartition := partition.(type) {
 	case doltTablePartition:
-		return newRowIterator(ctx, t, projCols, typedPartition)
+		return newRowIterator(ctx, t, sqlSch, projCols, typedPartition)
 	case index.SinglePartition:
-		return newRowIterator(ctx, t, projCols, doltTablePartition{rowData: typedPartition.RowData, end: NoUpperBound})
+		return newRowIterator(ctx, t, sqlSch, projCols, doltTablePartition{rowData: typedPartition.RowData, end: NoUpperBound})
 	}
 
 	return nil, errors.New("unsupported partition type")
@@ -958,13 +1003,81 @@ func (t *AlterableDoltTable) ShouldRewriteTable(
 	ctx *sql.Context,
 	oldSchema sql.PrimaryKeySchema,
 	newSchema sql.PrimaryKeySchema,
-	modifiedColumn *sql.Column,
+	oldColumn *sql.Column,
+	newColumn *sql.Column,
 ) bool {
-	// TODO: this could be a lot more specific, we don't always need to rewrite on schema changes in the new format
-	return types.IsFormat_DOLT_1(t.nbf) || len(oldSchema.Schema) < len(newSchema.Schema)
+	return t.isIncompatibleTypeChange(oldColumn, newColumn) ||
+		orderChanged(oldSchema, newSchema, oldColumn, newColumn) ||
+		isColumnDrop(oldSchema, newSchema) ||
+		isPrimaryKeyChange(oldSchema, newSchema)
 }
 
-func (t *AlterableDoltTable) RewriteInserter(ctx *sql.Context, newSchema sql.PrimaryKeySchema) (sql.RowInserter, error) {
+func orderChanged(oldSchema, newSchema sql.PrimaryKeySchema, oldColumn, newColumn *sql.Column) bool {
+	if oldColumn == nil || newColumn == nil {
+		return false
+	}
+
+	return oldSchema.Schema.IndexOfColName(oldColumn.Name) != newSchema.Schema.IndexOfColName(newColumn.Name)
+}
+
+func (t *AlterableDoltTable) isIncompatibleTypeChange(oldColumn *sql.Column, newColumn *sql.Column) bool {
+	if oldColumn == nil || newColumn == nil {
+		return false
+	}
+
+	existingCol, _ := t.sch.GetAllCols().GetByNameCaseInsensitive(oldColumn.Name)
+	newCol, err := sqlutil.ToDoltCol(schema.SystemTableReservedMin, newColumn)
+	if err != nil {
+		panic(err) // should be impossible, we check compatibility before this point
+	}
+
+	if !existingCol.TypeInfo.Equals(newCol.TypeInfo) {
+		if types.IsFormat_DOLT_1(t.Format()) {
+			// This is overly broad, we could narrow this down a bit
+			return true
+		}
+		if existingCol.Kind != newCol.Kind {
+			return true
+		} else if schema.IsColSpatialType(newCol) {
+			// TODO: we need to do this because some spatial type changes require a full table check, but not all.
+			//  We could narrow this check down.
+			return true
+		}
+	}
+
+	return false
+}
+
+func isColumnDrop(oldSchema sql.PrimaryKeySchema, newSchema sql.PrimaryKeySchema) bool {
+	return len(oldSchema.Schema) > len(newSchema.Schema)
+}
+
+func getDroppedColumn(oldSchema sql.PrimaryKeySchema, newSchema sql.PrimaryKeySchema) *sql.Column {
+	for _, col := range oldSchema.Schema {
+		if newSchema.IndexOf(col.Name, col.Source) < 0 {
+			return col
+		}
+	}
+	return nil
+}
+
+func isPrimaryKeyChange(oldSchema sql.PrimaryKeySchema,
+	newSchema sql.PrimaryKeySchema) bool {
+	return len(newSchema.PkOrdinals) != len(oldSchema.PkOrdinals)
+}
+
+func (t *AlterableDoltTable) RewriteInserter(
+	ctx *sql.Context,
+	oldSchema sql.PrimaryKeySchema,
+	newSchema sql.PrimaryKeySchema,
+	oldColumn *sql.Column,
+	newColumn *sql.Column,
+) (sql.RowInserter, error) {
+	err := validateSchemaChange(t.Name(), oldSchema, newSchema, oldColumn, newColumn)
+	if err != nil {
+		return nil, err
+	}
+
 	sess := dsess.DSessFromSess(ctx.Session)
 
 	// Begin by creating a new table with the same name and the new schema, then removing all its existing rows
@@ -1004,12 +1117,40 @@ func (t *AlterableDoltTable) RewriteInserter(ctx *sql.Context, newSchema sql.Pri
 		return nil, err
 	}
 
-	newSch, err = schema.Adapt(oldSch, newSch) // improvise, overcome
-	if err != nil {
-		return nil, err
+	newSch = schema.CopyChecksConstraints(oldSch, newSch)
+
+	if isColumnDrop(oldSchema, newSchema) {
+		newSch = schema.CopyIndexes(oldSch, newSch)
+		droppedCol := getDroppedColumn(oldSchema, newSchema)
+		for _, index := range newSch.Indexes().IndexesWithColumn(droppedCol.Name) {
+			_, err = newSch.Indexes().RemoveIndex(index.Name())
+			if err != nil {
+				return nil, err
+			}
+		}
+	} else if newColumn != nil && oldColumn != nil { // modify column
+		// It may be possible to optimize this and not always rewrite every index, but since we're already truncating the
+		// table to rewrite it we also truncate all the indexes. Much easier to get right.
+		for _, index := range oldSch.Indexes().AllIndexes() {
+			var colNames []string
+			for _, colName := range index.ColumnNames() {
+				if strings.ToLower(oldColumn.Name) == strings.ToLower(colName) {
+					colNames = append(colNames, newColumn.Name)
+				} else {
+					colNames = append(colNames, colName)
+				}
+			}
+			newSch.Indexes().AddIndexByColNames(index.Name(), colNames, schema.IndexProperties{
+				IsUnique:      index.IsUnique(),
+				IsUserDefined: index.IsUserDefined(),
+				Comment:       index.Comment(),
+			})
+		}
+	} else {
+		newSch = schema.CopyIndexes(oldSch, newSch)
 	}
 
-	// If we have an auto increment column, we need to set it here before we begin the rewrite process
+	// If we have an auto increment column, we need to set it here before we begin the rewrite process (it may have changed)
 	if schema.HasAutoIncrement(newSch) {
 		newSch.GetAllCols().Iter(func(tag uint64, col schema.Column) (stop bool, err error) {
 			if col.AutoIncrement {
@@ -1035,6 +1176,13 @@ func (t *AlterableDoltTable) RewriteInserter(ctx *sql.Context, newSchema sql.Pri
 		return nil, err
 	}
 
+	if len(oldSchema.PkOrdinals) > 0 && len(newSchema.PkOrdinals) == 0 {
+		newRoot, err = t.adjustForeignKeysForDroppedPk(ctx, newRoot)
+		if err != nil {
+			return nil, err
+		}
+	}
+
 	newWs := ws.WithWorkingRoot(newRoot)
 
 	// TODO: figure out locking. Other DBs automatically lock a table during this kind of operation, we should probably
@@ -1053,6 +1201,55 @@ func (t *AlterableDoltTable) RewriteInserter(ctx *sql.Context, newSchema sql.Pri
 	return ed, nil
 }
 
+// validateSchemaChange returns an error if the schema change given is not legal
+func validateSchemaChange(
+	tableName string,
+	oldSchema sql.PrimaryKeySchema,
+	newSchema sql.PrimaryKeySchema,
+	oldColumn *sql.Column,
+	newColumn *sql.Column,
+) error {
+	if newColumn != nil {
+		newCol, err := sqlutil.ToDoltCol(schema.SystemTableReservedMin, newColumn)
+		if err != nil {
+			panic(err)
+		}
+
+		if newCol.IsPartOfPK && schema.IsColSpatialType(newCol) {
+			return schema.ErrUsingSpatialKey.New(tableName)
+		}
+	}
+	return nil
+}
+
+func (t *AlterableDoltTable) adjustForeignKeysForDroppedPk(ctx *sql.Context, root *doltdb.RootValue) (*doltdb.RootValue, error) {
+	if t.autoIncCol.AutoIncrement {
+		return nil, sql.ErrWrongAutoKey.New()
+	}
+
+	fkc, err := root.GetForeignKeyCollection(ctx)
+	if err != nil {
+		return nil, err
+	}
+
+	fkcUpdates, err := backupFkcIndexesForPkDrop(ctx, t.sch, fkc)
+	if err != nil {
+		return nil, err
+	}
+
+	err = fkc.UpdateIndexes(ctx, t.sch, fkcUpdates)
+	if err != nil {
+		return nil, err
+	}
+
+	root, err = root.PutForeignKeyCollection(ctx, fkc)
+	if err != nil {
+		return nil, err
+	}
+
+	return root, nil
+}
+
 // DropColumn implements sql.AlterableTable
 func (t *AlterableDoltTable) DropColumn(ctx *sql.Context, columnName string) error {
 	if types.IsFormat_DOLT_1(t.nbf) {
@@ -1168,12 +1365,9 @@ func (t *AlterableDoltTable) dropColumnData(ctx *sql.Context, updatedTable *dolt
 	return updatedTable.UpdateNomsRows(ctx, newMapData)
 }
 
-// ModifyColumn implements sql.AlterableTable
+// ModifyColumn implements sql.AlterableTable. ModifyColumn operations are only used for operations that change only
+// the schema of a table, not the data. For those operations, |RewriteInserter| is used.
 func (t *AlterableDoltTable) ModifyColumn(ctx *sql.Context, columnName string, column *sql.Column, order *sql.ColumnOrder) error {
-	if types.IsFormat_DOLT_1(t.nbf) {
-		return nil
-	}
-
 	ws, err := t.db.GetWorkingSet(ctx)
 	if err != nil {
 		return err
@@ -1200,6 +1394,7 @@ func (t *AlterableDoltTable) ModifyColumn(ctx *sql.Context, columnName string, c
 		return err
 	}
 
+	// TODO: move this logic into ShouldRewrite
 	if !existingCol.TypeInfo.Equals(col.TypeInfo) {
 		if existingCol.Kind != col.Kind { // We only change the tag when the underlying Noms kind changes
 			tags, err := root.GenerateTagsForNewColumns(ctx, t.tableName, []string{col.Name}, []types.NomsKind{col.Kind}, nil)
@@ -1213,56 +1408,18 @@ func (t *AlterableDoltTable) ModifyColumn(ctx *sql.Context, columnName string, c
 		}
 	}
 
-	updatedTable, err := modifyColumn(ctx, table, existingCol, col, order, t.opts)
+	updatedTable, err := modifyColumn(ctx, table, existingCol, col, order)
 	if err != nil {
 		return err
 	}
 
 	// For auto columns modified to be auto increment, we have more work to do
 	if !existingCol.AutoIncrement && col.AutoIncrement {
-		updatedSch, err := updatedTable.GetSchema(ctx)
+		seq, err := t.getFirstAutoIncrementValue(ctx, columnName, column.Type, updatedTable)
 		if err != nil {
 			return err
 		}
 
-		rowData, err := updatedTable.GetRowData(ctx)
-		if err != nil {
-			return err
-		}
-
-		// Note that we aren't calling the public PartitionRows, because it always gets the table data from the session
-		// root, which hasn't been updated yet
-		rowIter, err := partitionRows(ctx, updatedTable, t.projectedCols, index.SinglePartition{RowData: rowData})
-		if err != nil {
-			return err
-		}
-
-		initialValue := column.Type.Zero()
-		colIdx := updatedSch.GetAllCols().IndexOf(columnName)
-
-		for {
-			r, err := rowIter.Next(ctx)
-			if err == io.EOF {
-				break
-			} else if err != nil {
-				return err
-			}
-
-			cmp, err := column.Type.Compare(initialValue, r[colIdx])
-			if err != nil {
-				return err
-			}
-			if cmp < 0 {
-				initialValue = r[colIdx]
-			}
-		}
-
-		seq, err := globalstate.CoerceAutoIncrementValue(initialValue)
-		if err != nil {
-			return err
-		}
-		seq++
-
 		updatedTable, err = updatedTable.SetAutoIncrementValue(ctx, seq)
 		if err != nil {
 			return err
@@ -1272,6 +1429,8 @@ func (t *AlterableDoltTable) ModifyColumn(ctx *sql.Context, columnName string, c
 		if err != nil {
 			return err
 		}
+
+		// TODO: this isn't transactional, and it should be
 		ait.AddNewTable(t.tableName)
 		ait.Set(t.tableName, seq)
 	}
@@ -1293,6 +1452,61 @@ func (t *AlterableDoltTable) ModifyColumn(ctx *sql.Context, columnName string, c
 	// return t.updateFromRoot(ctx, newRoot)
 }
 
+// getFirstAutoIncrementValue returns the next auto increment value for a table that just acquired one through an
+// ALTER statement.
+// TODO: this could use an index and avoid a full table scan in many cases
+func (t *AlterableDoltTable) getFirstAutoIncrementValue(
+	ctx *sql.Context,
+	columnName string,
+	columnType sql.Type,
+	table *doltdb.Table,
+) (uint64, error) {
+	updatedSch, err := table.GetSchema(ctx)
+	if err != nil {
+		return 0, err
+	}
+
+	rowData, err := table.GetRowData(ctx)
+	if err != nil {
+		return 0, err
+	}
+
+	// Note that we aren't calling the public PartitionRows, because it always gets the table data from the session
+	// root, which hasn't been updated yet
+	rowIter, err := partitionRows(ctx, table, t.sqlSch.Schema, t.projectedCols, index.SinglePartition{RowData: rowData})
+	if err != nil {
+		return 0, err
+	}
+
+	initialValue := columnType.Zero()
+	colIdx := updatedSch.GetAllCols().IndexOf(columnName)
+
+	for {
+		r, err := rowIter.Next(ctx)
+		if err == io.EOF {
+			break
+		} else if err != nil {
+			return 0, err
+		}
+
+		cmp, err := columnType.Compare(initialValue, r[colIdx])
+		if err != nil {
+			return 0, err
+		}
+		if cmp < 0 {
+			initialValue = r[colIdx]
+		}
+	}
+
+	seq, err := globalstate.CoerceAutoIncrementValue(initialValue)
+	if err != nil {
+		return 0, err
+	}
+	seq++
+
+	return seq, nil
+}
+
 func increment(val types.Value) types.Value {
 	switch val := val.(type) {
 	case types.Int:
@@ -1384,10 +1598,6 @@ func (t *AlterableDoltTable) CreateIndex(
 
 // DropIndex implements sql.IndexAlterableTable
 func (t *AlterableDoltTable) DropIndex(ctx *sql.Context, indexName string) error {
-	if types.IsFormat_DOLT_1(t.nbf) {
-		return nil
-	}
-
 	// We disallow removing internal dolt_ tables from SQL directly
 	if strings.HasPrefix(indexName, "dolt_") {
 		return fmt.Errorf("dolt internal indexes may not be dropped")
@@ -2188,12 +2398,22 @@ func (t *AlterableDoltTable) DropPrimaryKey(ctx *sql.Context) error {
 		return err
 	}
 
-	fkcUpdates, err := t.backupFkcIndexesForPkDrop(ctx, root)
+	fkc, err := root.GetForeignKeyCollection(ctx)
 	if err != nil {
 		return err
 	}
 
-	newRoot, err := t.updateFkcIndex(ctx, root, fkcUpdates)
+	fkcUpdates, err := backupFkcIndexesForPkDrop(ctx, t.sch, fkc)
+	if err != nil {
+		return err
+	}
+
+	err = fkc.UpdateIndexes(ctx, t.sch, fkcUpdates)
+	if err != nil {
+		return err
+	}
+
+	newRoot, err := root.PutForeignKeyCollection(ctx, fkc)
 	if err != nil {
 		return err
 	}
@@ -2222,109 +2442,6 @@ func (t *AlterableDoltTable) DropPrimaryKey(ctx *sql.Context) error {
 	return t.updateFromRoot(ctx, newRoot)
 }
 
-type fkIndexUpdate struct {
-	fkName  string
-	fromIdx string
-	toIdx   string
-}
-
-// updateFkcIndex applies a list of fkIndexUpdates to a ForeignKeyCollection and returns a new root value
-func (t *AlterableDoltTable) updateFkcIndex(ctx *sql.Context, root *doltdb.RootValue, updates []fkIndexUpdate) (*doltdb.RootValue, error) {
-	fkc, err := root.GetForeignKeyCollection(ctx)
-	if err != nil {
-		return nil, err
-	}
-
-	for _, u := range updates {
-		fk, ok := fkc.GetByNameCaseInsensitive(u.fkName)
-		if !ok {
-			return nil, errors.New("foreign key not found")
-		}
-		fkc.RemoveKeys(fk)
-		fk.ReferencedTableIndex = u.toIdx
-		fkc.AddKeys(fk)
-		err := fk.ValidateReferencedTableSchema(t.sch)
-		if err != nil {
-			return nil, err
-		}
-	}
-
-	root, err = root.PutForeignKeyCollection(ctx, fkc)
-	if err != nil {
-		return nil, err
-	}
-	return root, nil
-}
-
-// backupFkcIndexesForKeyDrop finds backup indexes to cover foreign key references during a primary
-// key drop. If multiple indexes are valid, we sort by unique and select the first.
-// This will not work with a non-pk index drop without an additional index filter argument.
-func (t *AlterableDoltTable) backupFkcIndexesForPkDrop(ctx *sql.Context, root *doltdb.RootValue) ([]fkIndexUpdate, error) {
-	fkc, err := root.GetForeignKeyCollection(ctx)
-	if err != nil {
-		return nil, err
-	}
-
-	indexes := t.sch.Indexes().AllIndexes()
-	if err != nil {
-		return nil, err
-	}
-
-	// pkBackups is a mapping from the table's PK tags to potentially compensating indexes
-	pkBackups := make(map[uint64][]schema.Index, len(t.sch.GetPKCols().TagToIdx))
-	for tag, _ := range t.sch.GetPKCols().TagToIdx {
-		pkBackups[tag] = nil
-	}
-
-	// prefer unique key backups
-	sort.Slice(indexes[:], func(i, j int) bool {
-		return indexes[i].IsUnique() && !indexes[j].IsUnique()
-	})
-
-	for _, idx := range indexes {
-		if !idx.IsUserDefined() {
-			continue
-		}
-
-		for _, tag := range idx.AllTags() {
-			if _, ok := pkBackups[tag]; ok {
-				pkBackups[tag] = append(pkBackups[tag], idx)
-			}
-		}
-	}
-
-	fkUpdates := make([]fkIndexUpdate, 0)
-	for _, fk := range fkc.AllKeys() {
-		// check if this FK references a parent PK tag we are trying to change
-		if backups, ok := pkBackups[fk.ReferencedTableColumns[0]]; ok {
-			covered := false
-			for _, idx := range backups {
-				idxTags := idx.AllTags()
-				if len(fk.TableColumns) > len(idxTags) {
-					continue
-				}
-				failed := false
-				for i := 0; i < len(fk.ReferencedTableColumns); i++ {
-					if idxTags[i] != fk.ReferencedTableColumns[i] {
-						failed = true
-						break
-					}
-				}
-				if failed {
-					continue
-				}
-				fkUpdates = append(fkUpdates, fkIndexUpdate{fk.Name, fk.ReferencedTableIndex, idx.Name()})
-				covered = true
-				break
-			}
-			if !covered {
-				return nil, sql.ErrCantDropIndex.New("PRIMARY")
-			}
-		}
-	}
-	return fkUpdates, nil
-}
-
 func findIndexWithPrefix(sch schema.Schema, prefixCols []string) (schema.Index, bool, error) {
 	type idxWithLen struct {
 		schema.Index
