@@ -26,9 +26,11 @@ import (
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/index"
 	"github.com/dolthub/dolt/go/libraries/doltcore/sqle/sqlutil"
 	"github.com/dolthub/dolt/go/store/pool"
+	"github.com/dolthub/dolt/go/store/prolly"
 	"github.com/dolthub/dolt/go/store/val"
 )
 
+// todo(andy): get from NodeStore
 var sharePool = pool.NewBuffPool()
 
 type prollyTableWriter struct {
@@ -36,7 +38,7 @@ type prollyTableWriter struct {
 	dbName    string
 
 	primary   indexWriter
-	secondary []prollyIndexWriter
+	secondary map[string]indexWriter
 
 	tbl    *doltdb.Table
 	sch    schema.Schema
@@ -52,17 +54,18 @@ type prollyTableWriter struct {
 
 var _ TableWriter = &prollyTableWriter{}
 
-func getSecondaryProllyIndexWriters(ctx context.Context, t *doltdb.Table, sqlSch sql.Schema, sch schema.Schema) ([]prollyIndexWriter, error) {
+func getSecondaryProllyIndexWriters(ctx context.Context, t *doltdb.Table, sqlSch sql.Schema, sch schema.Schema) (map[string]indexWriter, error) {
 	s, err := t.GetIndexSet(ctx)
 	if err != nil {
 		return nil, err
 	}
 
 	definitions := sch.Indexes().AllIndexes()
-	writers := make([]prollyIndexWriter, len(definitions))
+	writers := make(map[string]indexWriter)
 
-	for i, def := range definitions {
-		idxRows, err := s.GetIndex(ctx, sch, def.Name())
+	for _, def := range definitions {
+		defName := def.Name()
+		idxRows, err := s.GetIndex(ctx, sch, defName)
 		if err != nil {
 			return nil, err
 		}
@@ -71,8 +74,8 @@ func getSecondaryProllyIndexWriters(ctx context.Context, t *doltdb.Table, sqlSch
 		keyMap, valMap := ordinalMappingsFromSchema(sqlSch, def.Schema())
 		keyDesc, valDesc := m.Descriptors()
 
-		writers[i] = prollyIndexWriter{
-			name:   def.Name(),
+		writers[defName] = prollyIndexWriter{
+			name:   defName,
 			mut:    m.Mutate(),
 			keyBld: val.NewTupleBuilder(keyDesc),
 			keyMap: keyMap,
@@ -84,8 +87,51 @@ func getSecondaryProllyIndexWriters(ctx context.Context, t *doltdb.Table, sqlSch
 	return writers, nil
 }
 
+func getSecondaryKeylessProllyWriters(ctx context.Context, t *doltdb.Table, sqlSch sql.Schema, sch schema.Schema, primary prollyKeylessWriter) (map[string]indexWriter, error) {
+	s, err := t.GetIndexSet(ctx)
+	if err != nil {
+		return nil, err
+	}
+
+	definitions := sch.Indexes().AllIndexes()
+	writers := make(map[string]indexWriter)
+
+	for _, def := range definitions {
+		defName := def.Name()
+		idxRows, err := s.GetIndex(ctx, sch, defName)
+		if err != nil {
+			return nil, err
+		}
+		m := durable.ProllyMapFromIndex(idxRows)
+		m = prolly.ConvertToKeylessIndex(m)
+
+		keyMap, valMap := ordinalMappingsFromSchema(sqlSch, def.Schema())
+		keyDesc, valDesc := m.Descriptors()
+
+		writers[defName] = prollyKeylessSecondaryWriter{
+			name:    defName,
+			mut:     m.Mutate(),
+			primary: primary,
+			unique:  def.IsUnique(),
+			keyBld:  val.NewTupleBuilder(keyDesc),
+			keyMap:  keyMap,
+			valBld:  val.NewTupleBuilder(valDesc),
+			valMap:  valMap,
+		}
+	}
+
+	return writers, nil
+}
+
 // Insert implements TableWriter.
-func (w *prollyTableWriter) Insert(ctx *sql.Context, sqlRow sql.Row) error {
+func (w *prollyTableWriter) Insert(ctx *sql.Context, sqlRow sql.Row) (err error) {
+	if sqlRow, err = index.NormalizeRow(w.sqlSch, sqlRow); err != nil {
+		return err
+	}
+
+	if err := w.primary.Insert(ctx, sqlRow); err != nil {
+		return err
+	}
 	for _, wr := range w.secondary {
 		if err := wr.Insert(ctx, sqlRow); err != nil {
 			if sql.ErrUniqueKeyViolation.Is(err) {
@@ -94,14 +140,15 @@ func (w *prollyTableWriter) Insert(ctx *sql.Context, sqlRow sql.Row) error {
 			return err
 		}
 	}
-	if err := w.primary.Insert(ctx, sqlRow); err != nil {
-		return err
-	}
 	return nil
 }
 
 // Delete implements TableWriter.
-func (w *prollyTableWriter) Delete(ctx *sql.Context, sqlRow sql.Row) error {
+func (w *prollyTableWriter) Delete(ctx *sql.Context, sqlRow sql.Row) (err error) {
+	if sqlRow, err = index.NormalizeRow(w.sqlSch, sqlRow); err != nil {
+		return err
+	}
+
 	for _, wr := range w.secondary {
 		if err := wr.Delete(ctx, sqlRow); err != nil {
 			return err
@@ -115,6 +162,13 @@ func (w *prollyTableWriter) Delete(ctx *sql.Context, sqlRow sql.Row) error {
 
 // Update implements TableWriter.
 func (w *prollyTableWriter) Update(ctx *sql.Context, oldRow sql.Row, newRow sql.Row) (err error) {
+	if oldRow, err = index.NormalizeRow(w.sqlSch, oldRow); err != nil {
+		return err
+	}
+	if newRow, err = index.NormalizeRow(w.sqlSch, newRow); err != nil {
+		return err
+	}
+
 	for _, wr := range w.secondary {
 		if err := wr.Update(ctx, oldRow, newRow); err != nil {
 			if sql.ErrUniqueKeyViolation.Is(err) {
@@ -207,17 +261,26 @@ func (w *prollyTableWriter) Reset(ctx context.Context, sess *prollyWriteSession,
 	}
 	aiCol := autoIncrementColFromSchema(sch)
 	var newPrimary indexWriter
-	if _, ok := w.primary.(prollyKeylessWriter); ok {
-		newPrimary, err = getKeylessProllyWriter(ctx, tbl, sqlSch.Schema, sch)
+
+	var newSecondaries map[string]indexWriter
+	if schema.IsKeyless(sch) {
+		newPrimary, err = getPrimaryKeylessProllyWriter(ctx, tbl, sqlSch.Schema, sch)
+		if err != nil {
+			return err
+		}
+		newSecondaries, err = getSecondaryKeylessProllyWriters(ctx, tbl, sqlSch.Schema, sch, newPrimary.(prollyKeylessWriter))
+		if err != nil {
+			return err
+		}
 	} else {
 		newPrimary, err = getPrimaryProllyWriter(ctx, tbl, sqlSch.Schema, sch)
-	}
-	if err != nil {
-		return err
-	}
-	newSecondaries, err := getSecondaryProllyIndexWriters(ctx, tbl, sqlSch.Schema, sch)
-	if err != nil {
-		return err
+		if err != nil {
+			return err
+		}
+		newSecondaries, err = getSecondaryProllyIndexWriters(ctx, tbl, sqlSch.Schema, sch)
+		if err != nil {
+			return err
+		}
 	}
 
 	w.tbl = tbl
@@ -233,12 +296,12 @@ func (w *prollyTableWriter) Reset(ctx context.Context, sess *prollyWriteSession,
 
 func (w *prollyTableWriter) table(ctx context.Context) (t *doltdb.Table, err error) {
 	// flush primary row storage
-	m, err := w.primary.Map(ctx)
+	pm, err := w.primary.Map(ctx)
 	if err != nil {
 		return nil, err
 	}
 
-	t, err = w.tbl.UpdateRows(ctx, durable.IndexFromProllyMap(m))
+	t, err = w.tbl.UpdateRows(ctx, durable.IndexFromProllyMap(pm))
 	if err != nil {
 		return nil, err
 	}
@@ -249,14 +312,14 @@ func (w *prollyTableWriter) table(ctx context.Context) (t *doltdb.Table, err err
 		return nil, err
 	}
 
-	for _, wr := range w.secondary {
-		m, err := wr.mut.Map(ctx)
+	for _, wrSecondary := range w.secondary {
+		sm, err := wrSecondary.Map(ctx)
 		if err != nil {
 			return nil, err
 		}
-		idx := durable.IndexFromProllyMap(m)
+		idx := durable.IndexFromProllyMap(sm)
 
-		s, err = s.PutIndex(ctx, wr.name, idx)
+		s, err = s.PutIndex(ctx, wrSecondary.Name(), idx)
 		if err != nil {
 			return nil, err
 		}
@@ -279,9 +342,6 @@ func (w *prollyTableWriter) table(ctx context.Context) (t *doltdb.Table, err err
 }
 
 func (w *prollyTableWriter) flush(ctx *sql.Context) error {
-	if !w.primary.HasEdits(ctx) {
-		return nil
-	}
 	ws, err := w.flusher.Flush(ctx)
 	if err != nil {
 		return err
