@@ -279,8 +279,10 @@ type Config struct {
 	TestOptions []string
 	// ScriptDir is a path to a directory of lua scripts
 	ScriptDir string
-	// DirtyClone downloads a database with existing chunks and commits
+	// InitBigRepo downloads a database with existing chunks and commits
 	InitBigRepo bool
+	// NomsBinFormat specifies the NomsBinFormat
+	NomsBinFormat string
 }
 
 // NewConfig returns a new Config
