@@ -20,7 +20,8 @@ flatc -o $GEN_DIR --gen-onefile --filename-suffix "" --gen-mutable --go-namespac
   storeroot.fbs \
   table.fbs \
   tag.fbs \
-  workingset.fbs
+  workingset.fbs \
+  encoding.fbs
 
 # prefix files with copyright header
 for FILE in $GEN_DIR/*.go;
