@@ -103,12 +103,11 @@ func runSync(ctx context.Context, args []string) int {
 	nonFF := false
 	srcCS := datas.ChunkStoreFromDatabase(sourceStore)
 	sinkCS := datas.ChunkStoreFromDatabase(sinkDB)
-	wrf, err := types.WalkRefsForChunkStore(srcCS)
-	util.CheckError(err)
+	waf := types.WalkAddrsForNBF(sourceVRW.Format())
 	f := func() error {
 		defer profile.MaybeStartProfile().Stop()
 		addr := sourceRef.TargetHash()
-		err := pull.Pull(ctx, srcCS, sinkCS, wrf, addr, progressCh)
+		err := pull.Pull(ctx, srcCS, sinkCS, waf, addr, progressCh)
 
 		if err != nil {
 			return err
