@@ -23,6 +23,10 @@ import (
 	"github.com/dolthub/dolt/go/store/val"
 )
 
+const MessageTypesKind int = 28
+
+const messagePrefixSz = 3
+
 type Message []byte
 
 type Serializer interface {
@@ -30,7 +34,7 @@ type Serializer interface {
 }
 
 func GetKeysAndValues(msg Message) (keys, values val.SlicedBuffer, cnt uint16) {
-	id := serial.GetFileID(msg)
+	id := serial.GetFileID(msg[messagePrefixSz:])
 
 	if id == serial.ProllyTreeNodeFileID {
 		return getProllyMapKeysAndValues(msg)
@@ -46,7 +50,7 @@ func GetKeysAndValues(msg Message) (keys, values val.SlicedBuffer, cnt uint16) {
 }
 
 func WalkAddresses(ctx context.Context, msg Message, cb func(ctx context.Context, addr hash.Hash) error) error {
-	id := serial.GetFileID(msg)
+	id := serial.GetFileID(msg[messagePrefixSz:])
 	switch id {
 	case serial.ProllyTreeNodeFileID:
 		return walkProllyMapAddresses(ctx, msg, cb)
@@ -58,7 +62,7 @@ func WalkAddresses(ctx context.Context, msg Message, cb func(ctx context.Context
 }
 
 func GetTreeLevel(msg Message) int {
-	id := serial.GetFileID(msg)
+	id := serial.GetFileID(msg[messagePrefixSz:])
 	switch id {
 	case serial.ProllyTreeNodeFileID:
 		return getProllyMapTreeLevel(msg)
@@ -70,7 +74,7 @@ func GetTreeLevel(msg Message) int {
 }
 
 func GetTreeCount(msg Message) int {
-	id := serial.GetFileID(msg)
+	id := serial.GetFileID(msg[messagePrefixSz:])
 	switch id {
 	case serial.ProllyTreeNodeFileID:
 		return getProllyMapTreeCount(msg)
@@ -82,7 +86,7 @@ func GetTreeCount(msg Message) int {
 }
 
 func GetSubtrees(msg Message) []uint64 {
-	id := serial.GetFileID(msg)
+	id := serial.GetFileID(msg[messagePrefixSz:])
 	switch id {
 	case serial.ProllyTreeNodeFileID:
 		return getProllyMapSubtrees(msg)
