@@ -17,6 +17,8 @@ package val
 import (
 	"time"
 
+	"github.com/shopspring/decimal"
+
 	"github.com/dolthub/dolt/go/store/pool"
 )
 
@@ -157,38 +159,63 @@ func (tb *TupleBuilder) PutFloat64(i int, v float64) {
 	tb.pos += float64Size
 }
 
-func (tb *TupleBuilder) PutTimestamp(i int, v time.Time) {
-	tb.Desc.expectEncoding(i, DateEnc, DatetimeEnc, TimestampEnc)
-	tb.fields[i] = tb.buf[tb.pos : tb.pos+timestampSize]
-	writeTimestamp(tb.fields[i], v)
-	tb.pos += timestampSize
+func (tb *TupleBuilder) PutBit(i int, v uint64) {
+	tb.Desc.expectEncoding(i, Bit64Enc)
+	tb.fields[i] = tb.buf[tb.pos : tb.pos+bit64Size]
+	writeBit64(tb.fields[i], v)
+	tb.pos += bit64Size
 }
 
-// PutSqlTime writes a string to the ith field of the Tuple being built.
-func (tb *TupleBuilder) PutSqlTime(i int, v string) {
-	tb.Desc.expectEncoding(i, TimeEnc)
-	sz := ByteSize(len(v)) + 1
+func (tb *TupleBuilder) PutDecimal(i int, v decimal.Decimal) {
+	tb.Desc.expectEncoding(i, DecimalEnc)
+	sz := sizeOfDecimal(v)
 	tb.fields[i] = tb.buf[tb.pos : tb.pos+sz]
-	writeString(tb.fields[i], v)
+	writeDecimal(tb.fields[i], v)
 	tb.pos += sz
 }
 
 // PutYear writes an int16-encoded year to the ith field of the Tuple being built.
 func (tb *TupleBuilder) PutYear(i int, v int16) {
-	// todo(andy): yearSize, etc?
 	tb.Desc.expectEncoding(i, YearEnc)
-	tb.fields[i] = tb.buf[tb.pos : tb.pos+int16Size]
-	writeInt16(tb.fields[i], v)
+	tb.fields[i] = tb.buf[tb.pos : tb.pos+yearSize]
+	writeYear(tb.fields[i], v)
 	tb.pos += int16Size
 }
 
-func (tb *TupleBuilder) PutDecimal(i int, v string) {
-	tb.Desc.expectEncoding(i, DecimalEnc)
-	// todo(andy): temporary implementation
-	sz := ByteSize(len(v)) + 1
-	tb.fields[i] = tb.buf[tb.pos : tb.pos+sz]
-	writeString(tb.fields[i], v)
-	tb.pos += sz
+func (tb *TupleBuilder) PutDate(i int, v time.Time) {
+	tb.Desc.expectEncoding(i, DateEnc)
+	tb.fields[i] = tb.buf[tb.pos : tb.pos+dateSize]
+	writeDate(tb.fields[i], v)
+	tb.pos += dateSize
+}
+
+// PutSqlTime writes a string to the ith field of the Tuple being built.
+func (tb *TupleBuilder) PutSqlTime(i int, v int64) {
+	tb.Desc.expectEncoding(i, TimeEnc)
+	tb.fields[i] = tb.buf[tb.pos : tb.pos+timeSize]
+	writeTime(tb.fields[i], v)
+	tb.pos += timeSize
+}
+
+func (tb *TupleBuilder) PutDatetime(i int, v time.Time) {
+	tb.Desc.expectEncoding(i, DatetimeEnc)
+	tb.fields[i] = tb.buf[tb.pos : tb.pos+datetimeSize]
+	writeDatetime(tb.fields[i], v)
+	tb.pos += datetimeSize
+}
+
+func (tb *TupleBuilder) PutEnum(i int, v uint16) {
+	tb.Desc.expectEncoding(i, EnumEnc)
+	tb.fields[i] = tb.buf[tb.pos : tb.pos+enumSize]
+	writeEnum(tb.fields[i], v)
+	tb.pos += enumSize
+}
+
+func (tb *TupleBuilder) PutSet(i int, v uint64) {
+	tb.Desc.expectEncoding(i, SetEnc)
+	tb.fields[i] = tb.buf[tb.pos : tb.pos+setSize]
+	writeSet(tb.fields[i], v)
+	tb.pos += setSize
 }
 
 // PutString writes a string to the ith field of the Tuple being built.
@@ -227,6 +254,21 @@ func (tb *TupleBuilder) PutGeometry(i int, v []byte) {
 	tb.pos += sz
 }
 
+// PutHash128 writes a hash128 to the ith field of the Tuple being built.
+func (tb *TupleBuilder) PutHash128(i int, v []byte) {
+	tb.Desc.expectEncoding(i, Hash128Enc)
+	tb.fields[i] = tb.buf[tb.pos : tb.pos+hash128Size]
+	writeHash128(tb.fields[i], v)
+	tb.pos += hash128Size
+}
+
+func (tb *TupleBuilder) PutAddress(i int, v []byte) {
+	tb.Desc.expectEncoding(i, AddressEnc)
+	tb.fields[i] = tb.buf[tb.pos : tb.pos+addressSize]
+	writeAddress(tb.fields[i], v)
+	tb.pos += addressSize
+}
+
 // PutRaw writes a []byte to the ith field of the Tuple being built.
 func (tb *TupleBuilder) PutRaw(i int, buf []byte) {
 	if buf == nil {
